
// *****************************************************************************
//
// source:    utstage.C
//
// created:    1 Jun 1999
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    utstage_
//
// called by: IFOPEN
//
// changes:
// 09 Sep 1999 (AG) replace cerr by cout
// 31 Mar 2000 (AG) fix new defines for g++
// 15 Sep 2000 (AG) added l4aFileF_()
// 06 Oct 2000 (AG) polished printout
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <string.h>

// ONLINE includes
#include "stage.h"

// ARTE includes
#include "farm/farm.hh"
#include "farm/fa.hh"

// externals
extern int  FADEB;
extern char *HBSTAGE;
extern struct {
  char ctape[256];
  char cdisk[256];
} stage_;

//
// program code
//
extern "C" void
utstage_()
{
  ARTE_mark( "utstage 0", 0 );

  // handle names
  char *ptr = strtok( stage_.ctape, " " ); // find end of tape name
  char tmp[256];                           // temporary tape name
  strcpy( tmp, ptr );                      // copy tape name
  memset( stage_.cdisk, 0, 256 );          // reset disk file name
  memset( stage_.ctape, 0, 256 );          // reset tape file name
  strcpy( stage_.ctape, tmp );             // copy clean tape name
  cout << "FILE: Tape file: " << stage_.ctape << endl;
  
  // search for disk file
  char *disk = l4aFileF_( stage_.ctape, FADEB );
  strcpy( stage_.cdisk, disk );
  if( FADEB ) cout << "FILE: Disk file: " << stage_.cdisk << endl;
  if( strlen( disk ) ){ 
    cout << "FILE: Disk file: " << stage_.cdisk << endl;
    ARTE_mark( "utstage 1", 0 );  
    return;
  }

#if ( defined linux ) || ( defined sun ) || ( defined sgi ) || ( defined __linux ) || ( defined __sun ) || ( defined __sgi )
  
  // staging implemented for Linux, Sun, SGI
  int rc = l4aStageF_( stage_.ctape, HBSTAGE, FADEB );
  if( rc == L4A_OK ){
    //cout << "STAGE: Tape file: " << stage_.ctape << endl;
  }
  else if( rc == L4A_FAILURE ){
    cout << "STAGE: ERROR: Staging FAILED for tape file: " << stage_.ctape << endl; 
    ARTE_mark( "utstage 1", 0 );
    return;
  }
  else if( rc == L4A_OSM_FAILURE ){
    cout << "STAGE: ERROR: OSM error for tape file: " << stage_.ctape << endl; 
    ARTE_mark( "utstage 1", 0 );
    return;
  }
  else if( rc == L4A_NO_ACS_FILE ){
    cout << "STAGE: ERROR: No such tape file: " << stage_.ctape << endl;
    ARTE_mark( "utstage 1", 0 );
    return;
  }
  else {
    cout << "STAGE: ERROR: While staging tape file: " << stage_.ctape << " rc=" << rc << endl; 
    ARTE_mark( "utstage 1", 0 );
    return;
  }

  // tape of disk file
  memset( stage_.cdisk, 0, 256 );
  strcat( stage_.cdisk, HBSTAGE );
  strcat( stage_.cdisk, stage_.ctape + 4 ); // omit "/acs"

  //cout << "STAGE: Tape file: " << stage_.ctape << endl;
  cout << "STAGE: Disk file: " << stage_.cdisk << endl;
  
#else
  
  // staging NOT implemented
  cout << "STAGE: Staging NOT implemented for your machine!" << endl; 

#endif

  ARTE_mark( "utstage 1", 0 );  
}
