#ifndef VDS_CLASS_MGEDE_HH
#define VDS_CLASS_MGEDE_HH

using namespace std;

#include <iostream.h>
#include <cmath>
#include <vector>
#include <map>
#include <set>
#include <multimap.h>
#include "arte/arte.hh"
#include "Class_GEDE.hh"


  class MGede;
  class MGedeIterator;
  class MGedeZIterator;

  typedef set<int>                         Gede_S;
  typedef set<int>::iterator               Gede_SIt;
  typedef vector<Gede>                     Gede_V;
  typedef vector<Gede>::iterator           Gede_VIt;
  typedef map<int,    int>                 Gede_M;
  typedef map<int,    int>::iterator       Gede_MIt;
  typedef multimap<int, int>               Gede_zM;
  typedef multimap<int, int>::iterator     Gede_zMIt;

  typedef ArteTable<GEDE>::iterator ArteGedeIt;

  class MGede {
  private:
    friend class MGedeIterator;
    friend class MGedeZIterator;
    Gede_V     vgede;                // Vector which holds all gede objects
    Gede_S     sdeadgede;            // store Arte gede idx of dead planes
    Gede_M     index_map;            // sort gede according to gede->index
    Gede_zM    z_map;                // sort gede according to z 

    // forbid operator=
    //    const MGede& operator=(const MGede&);

    void init();
    void map_insert(const Gede& gede, const int pos);
    inline int z_key(double z) const;
    inline double round(double z) const;
  public:
    MGede();
    MGede(Gede gede);  // for testing
    MGede(ArteGedeIt begin, ArteGedeIt end);

    void push_back(const Gede& gede);
    void push_back(const int deadgede);
    int  size()  const;
    void index_order();
    void z_order();
    bool is_dead(int gede) const;

    //
    // Iterator definitions
    //
    typedef MGedeIterator iterator;

    iterator begin();
    iterator end();
    iterator lower_bound(int key);
    iterator upper_bound(int key);
    iterator find(int key);
    iterator lower_bound(unsigned int key);
    iterator upper_bound(unsigned int key);
    iterator find(unsigned int key);

    typedef MGedeZIterator ziterator;

    ziterator zbegin();
    ziterator zend();
    ziterator lower_bound(const double key);
    ziterator upper_bound(const double key);
    ziterator find(const double key);

    //
    // Overloaded operators
    //
    Gede& operator[](int index);
    Gede& operator[](double z);

    friend ostream& operator<<(ostream& s, MGede& a) {
      s << "Map size: " << a.size() << endl;
      copy(a.vgede.begin(), a.vgede.end(), ostream_iterator<Gede>(s,"\n"));
      return s;
    }
  }; //class MGede

  //
  // Class MGedeIterator
  //
  class MGedeIterator {
  private:
    friend class MGede;
    friend int operator==( const MGedeIterator&,
                           const MGedeIterator& );
    Gede_VIt   begin;
    Gede_MIt   iter;
  public:
    typedef MGedeIterator self;

    MGedeIterator();

    self& operator++();
    self& operator--();
    const self operator+(int n);
    Gede& operator*()  const;
    Gede* operator->() const;

  }; //class MGedeIterator

  //==============================================================================
  // operator==
  //==============================================================================
  inline int operator==(const MGedeIterator& it1,
                        const MGedeIterator& it2)
  { return it1.iter==it2.iter; }

  //
  // Class MGedeZIterator
  //
  class MGedeZIterator {
  private:
    friend class MGede;
    friend int operator==( const MGedeZIterator&,
                           const MGedeZIterator& );
    Gede_VIt    begin;
    Gede_zMIt   iter;
  public:
    typedef MGedeZIterator self;

    MGedeZIterator();

    self& operator++();
    self& operator--();
    const self operator+(int n);
    Gede& operator*()  const;
    Gede* operator->() const;

  }; //class MGedeZIterator

  //==============================================================================
  // operator==
  //==============================================================================
  inline int operator==(const MGedeZIterator& it1,
                        const MGedeZIterator& it2)
  { return it1.iter==it2.iter; }

#endif
