
/**********************************************************************/
//
// $Source: /afs/desy.de/group/hera-b/repository/ARTE/cluearte/cluearte.C,v $
// $Revision: 1.6 $
// $Author: natasha $
// $Date: 1999/11/02 17:44:58 $
//
// Created: 07-16-99,   C. Borgmeier
//
// clueWriteEvent() writes the current ARTE event into a CLUE file.
// The filename must be given by the kuip command
//
//   CLUE/SETFILENAME xxx
//
// before the first invocation.
//
/**********************************************************************/


#include <string>

#ifdef USE_ROOT
// ROOT includes
#include <TROOT.h>
#include <TFile.h>
#include <TTree.h>
#include <TBranch.h>
#endif

// CLUE
#include <Clue/RtEvent.h>
#include <Clue/ClTOPP.hh>
#include <Gate/ObjArray.hh>


extern "C" void kugets_( char*, int *, int );
extern "C" void kugeti_( int& );
extern "C" void clkuip_();
void atregkuip( void (*function)(), int priority );


#ifdef USE_ROOT
//
// replacement for ROOT\'s TTree::Branch to handle GClonesArray<T>
//
TBranch* clueBranch( TTree* thisObj,
		     const Text_t *name,
		     const Text_t *classname,
		     void *addobj,
		     Int_t bufsize,
		     Int_t splitlevel);
static TFile* cluefile = 0;


//
// this object writes the CLUE file on program termination
//
static class ClueCloser
{
public:
  ClueCloser()
    :fROOT( "recon", "recon" )
    {
      //if( !TROOT::Initialized() ) {
      //fROOT = new TROOT( "recon", "recon" );
      //}
    }
  ~ClueCloser()
    { if( cluefile!=0 ) cluefile->Write(); }
  
  TROOT fROOT;
} _clueCloser;
#endif /* USE_ROOT */


//
// this object initializes the CLUE kuip commands
//
static class ClueIniter
{
public:
  ClueIniter()
    {
      atregkuip( clkuip_, 10 );
    } 
} _clueIniter;


//
//
//


static std::string beefilename;

//
// cluesetfilename
//
extern "C" void cluesetfilename_()
{
  int len;
  int len1 = 512;
  char input[512];
  kugets_(input, &len, len1);
  
  beefilename = std::string( input, len );
}


//
//
//
extern "C" void cluestorersegflag_()
{
  int flag;
  kugeti_( flag );

  RtEvent::storeRSEG_Flag = flag;
}


RtEvent* clueEvent              = 0;
#ifdef USE_ROOT
static TTree* ttree         = 0;
static TTree* toppTree      = 0;
static int initialCompLevel = 2;


//
//
//
static int clueCreateFile()
{
  if( !beefilename.empty() ) {
    
    if( clueEvent==0 ) clueEvent = new RtEvent();
    
    cluefile = new TFile( beefilename.c_str(),"RECREATE",
			  "Clue ntuple");
    cluefile->SetCompressionLevel( initialCompLevel );
    
    ttree = new TTree("Clue","CLUE tree");
    //ttree->SetAutoSave(1000000);  // autosave when 1 Mbyte written
    Int_t split = 1;
    Int_t bufsize = 65536;
    if (split)  bufsize /= 4;
    
    clueBranch( ttree, "event", "RtEvent", &clueEvent, bufsize, split);
    
    cluefile->cd();
    toppTree = new TTree( "constants", "constants" );
    
    toppTree->Branch( "toppTable", &RtEvent::toppLink, bufsize, 1 );

    return 0;
  }
  return -1;
}
#endif


//
// cluesetbranchstatus
//
extern "C" void cluesetbranchstatus_()
{
  int len;
  int len1 = 511;
  char input[512];
  kugets_( input, &len, len1 );
  input[len] = 0;

  int flag;
  kugeti_( flag );

#ifdef USE_ROOT
  if( ttree==0 ) {
    if( clueCreateFile()!=0 || ttree==0 ) return;
  }
  
  ttree->SetBranchStatus( input, flag );
#endif
}


extern "C" void cluesetcompressionlevel_()
{
  int level;
  kugeti_( level );
  if( level < 0 || level > 9 ) {
    
  } else {
#ifdef USE_ROOT
    if( cluefile!=0 ) {
      cluefile->SetCompressionLevel( level );
    } else {
      initialCompLevel = level;
    }
#endif
  }
}


//
// write the current ARTE event
//
int clueWriteEvent()
{
  if( !beefilename.empty() ) {
#ifdef USE_ROOT
    static bool firstcall = true;

    if( firstcall ) {
      if( clueCreateFile()!=0 ) return -1;
    }
    
    clueEvent->init();
    
    if( firstcall ) {
      firstcall = false;
      toppTree->Fill();
    }
    ttree->Fill();
#endif
  }
  return 0;
}


//
// get the current ARTE event as RtEvent object
//
RtEvent* clueGetEvent()
{
  if( clueEvent==0 ) clueEvent = new RtEvent();
  clueEvent->init();
  return clueEvent;
}
