
// ********************************************************************
//
// Routine:      DQ_RECO_JPSIEE
// Author:       T.Oest
//
// Description:  fill Jpsi DQ histograms
//
// Changes:
//  5 Jul 2000 (AG) called it DQ_RECO_JPSIEE
//  7 Jul 2000 (AG) polished printout
//
// ********************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

// ONLINE includes
#include "rhp.h" 

// ARTE includes
#include "arte/RSEG_cmp.hh"
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"

// prototypes
float rc99Bremsstrahlung( ArtePointer<RTRA> );

//
// function code
//
int
DQ_RECO_JPSIEE_init()
{
  // show up
  ARTE_mark( "DQ_RECO_JPSIEE_init 0", 0 );
  cout << "DQ_RECO_JPSIEE_init" << endl;

  // histograms
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 100 ), 
	      "RECO JPSIEE candidates", 60, 2, 5 );
  
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 101 ), 
	      "RECO JPSIEE candidates (hard cuts)", 60, 2, 5 );
  
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 102 ), 
	      "RECO JPSIEE candidates (hard cuts, using E(ecal))", 60, 2, 5 );
  
  rhp_hbook2( DQ_ID( COMP_RECO,LVL_REC, 103 ), 
	      "RECO JPSIEE tx vs ty", 100, -0.2, 0.2,
	      100, -0.2, 0.2);
  
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 104 ), 
	      "RECO JPSIEE E over p ", 100, -2, 2 );
  
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 105 ),
	      "RECO JPSIEE E over p (pure sample)", 100, -2, 2 );
  
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 106 ), 
	      "RECO JPSIEE vertex z (pure sample)", 100, -15, 10 );

  // show up
  ARTE_mark( "DQ_RECO_JPSIEE_init 1", 0 );

  return 0;
}

//
// function code
//
int
DQ_RECO_JPSIEE_evnt()
{
  // show up
  ARTE_mark( "DQ_RECO_JPSIEE_evnt", 0 );

  //-----------------------------------------------------------------
  //   Jpsi tracks
  //-----------------------------------------------------------------
  
  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end(); ++rtra ) {
    
    if ((*rtra)->fit == 2 ){
      
      ArtePointer<RVER> rver = (*rtra)->ever;
      
      if (rver != 0 ){
	
	//-----------------------------------------------------------------
	//   get tracks from vertex
	//-----------------------------------------------------------------
	
	ArtePointer<RREL> rrel1 = *(rver->rrel.begin());
	ArtePointer<RREL> rrel2 = *(++(rver->rrel.begin()));
	ArtePointer<RTRA> rtra1 = rrel1->rtra;
	ArtePointer<RTRA> rtra2 = rrel2->rtra;
	
	//-----------------------------------------------------------------
	//   get segments and vertex
	//-----------------------------------------------------------------
	ArtePointer<RSEG> rVDS1  = 0;
	ArtePointer<RSEG> rRICH1 = 0;
	ArtePointer<RCCL> rccl1  = 0;
	
	if (rtra1->rccl.begin() != rtra1->rccl.end())
	  rccl1 = * rtra1->rccl.begin();
	for (ArteRelationIterator<RSEG> rseg2 = rtra1->rseg.begin();
	     rseg2 != rtra1->rseg.end(); ++rseg2){
	  
	  if ((*rseg2)->fit <= 0 ||  (*rseg2)->fit == 8 ||
	      (*rseg2)->fit == 10){
	    
	    if ((*rseg2)->cmp & Rsegc::vxd ) rVDS1  = *rseg2;
	    if ((*rseg2)->cmp & Rsegc::rich) rRICH1 = *rseg2;
	    
	  }
	}
	
	ArtePointer<RSEG> rVDS2  = 0;
	ArtePointer<RSEG> rRICH2 = 0;
	ArtePointer<RCCL> rccl2  = 0;
	
	if (rtra2->rccl.begin() != rtra2->rccl.end())
	  rccl2 = *rtra2->rccl.begin();
	
	for (ArteRelationIterator<RSEG> rseg2 = rtra2->rseg.begin();
	     rseg2 != rtra2->rseg.end(); ++rseg2){
	  
	  if ((*rseg2)->fit <= 0 ||  (*rseg2)->fit == 8 ||
	      (*rseg2)->fit == 10){
	    
	    if ((*rseg2)->cmp & Rsegc::vxd ) rVDS2  = *rseg2;
	    if ((*rseg2)->cmp & Rsegc::rich) rRICH2 = *rseg2;
	    
	  }
	}
	
	//-----------------------------------------------------------------
	//   kinematics
	//-----------------------------------------------------------------
	float p1, px1, py1, pz1, pt1;
	float p2, px2, py2, pz2, pt2;
	float px, py, pz, p, pt;
	float E1, E2, ep1, ep2, Eb1, Eb2, mass, mECAL;
	
	pz1 = 1./ fabs(rtra1->pf) /
	  sqrt(1.+rtra1->txf*rtra1->txf +
	       rtra1->tyf*rtra1->tyf);
	px1 = rtra1->txf * pz1;
	py1 = rtra1->tyf * pz1;
	pt1 = sqrt(px1*px1+py1*py1);
	p1  = 1./rtra1->pf;
	
	pz2 = 1./ fabs(rtra2->pf) /
	  sqrt(1.+rtra2->txf*rtra2->txf +
	       rtra2->tyf*rtra2->tyf);
	px2 = rtra2->txf * pz2;
	py2 = rtra2->tyf * pz2;
	pt2 = sqrt(px2*px2+py2*py2);
	p2  = 1./rtra2->pf;
	
	px  = px1 + px2;
	py  = py1 + py2;
	pz  = pz1 + pz2;
	p   = sqrt(px*px+py*py+pz*pz);
	pt  = sqrt(px*px+py*py);
	
	E1  = rccl1->esum;
	E2  = rccl2->esum;
	
	ep1 = E1 * rtra1->pf;
	ep2 = E2 * rtra2->pf;
	
	Eb1 = rc99Bremsstrahlung(rtra1);
	Eb2 = rc99Bremsstrahlung(rtra2);
	
	//-----------------------------------------------------------------
	//  cuts
	//-----------------------------------------------------------------
	
        float vchiCUT = 4.;
        float depCUT  = 0.3;
        float EbCUT   = 4.;
        float ptCUT   = 1.;
	
        if ( (rrel1->chi + rrel2->chi) < vchiCUT && 
             ( Eb1+Eb2 > EbCUT || ( rRICH1 != 0 && rRICH2 != 0 ))){
	  
	  
	  mass  = sqrt((1+Eb1/fabs(p1)) * (1+Eb2/fabs(p2))) * rver->mass;
	  mECAL = sqrt((E1+Eb1)/fabs(p1) * (E2+Eb2)/fabs(p2)) * rver->mass;
	  
	  //-----------------------------------------------------------------
	  //  J/psi sample
	  //-----------------------------------------------------------------
	  
	  if(  1 - depCUT < fabs(ep1) && fabs(ep1) < 1. + depCUT && 
	       1 - depCUT < fabs(ep2) && fabs(ep2) < 1. + depCUT &&
	       pt1 > ptCUT && pt2 > ptCUT &&
	       ( Eb1+Eb2 > EbCUT || (rRICH1 != 0 && rRICH2 != 0)) ){
	    
	    rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, 100) , mass, 1 );
	    
	    rhp_hfill(DQ_ID(COMP_RECO, LVL_REC, 103) ,rtra1->txf, rtra1->tyf ,1);
	    rhp_hfill(DQ_ID(COMP_RECO, LVL_REC, 103) ,rtra2->txf, rtra2->tyf ,1);
	    
	  }

	  //-----------------------------------------------------------------
	  //  plots for pure J/psi sample
	  //-----------------------------------------------------------------
	  
	  if ( Eb1+Eb2 > EbCUT &&  rRICH1 != 0 && rRICH2 != 0 &&
	       pt1 > ptCUT && pt2 > ptCUT ){
	    
	    if (mass <3.3 && mass > 2.8){
	      
	      if (1 - depCUT < fabs(ep1) && fabs(ep1) < 1. + depCUT)
		rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, 105) , ep2, 1 );
	      
	      if (1 - depCUT < fabs(ep2) && fabs(ep2) < 1. + depCUT)
		rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, 105) , ep1, 1 );
	      
	    }
	    
	    
	    if(  1 - depCUT < fabs(ep1) && fabs(ep1) < 1. + depCUT && 
		 1 - depCUT < fabs(ep2) && fabs(ep2) < 1. + depCUT ){
	      
	      rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, 101) , mass, 1 );
	      rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, 102) , mECAL, 1 );
	      if (mass <3.3 && mass > 2.8)
		rhp_hf1( DQ_ID(COMP_RECO, LVL_REC, 106) , rver->zv, 1 );
	    }
	  }
	}
      }
    }   
  }

  // show up
  ARTE_mark( "DQ_RECO_JPSIEE_evnt", 0 );

  return 0;
}
