// ********************************************************************
//
//  Routine:      gageFillRhpVdsEcalRich
//  Author:       T. Oest
//
//  Description:  fill segments pairs in rhp ring buffer or plot
//                for Vds ECALRICH
//
//  Modification:
//
// ********************************************************************

// includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

#include "arte/arte.hh"
#include "gage/gage.hh"
#include "arte/RSEG_cmp.hh"
#include "rhp.h"
#include "dq/DQ.h"

extern FILE * gageFile;


//*****************************************************************
// function
//*****************************************************************

void gageFillRhpVdsEcalRich(int cmpRSEG, int cmpAlign, float * Cuts){

if (cmpRSEG & cmpAlign) return;

//-----------------------------------------------------------------
//   Target position
//-----------------------------------------------------------------
    float xTARGET = 0.0, yTARGET = 0.0 , zTARGET = 0.0;
    float diff;

    for( ArteTable<GTAR>::iterator gtar = ArteTable<GTAR>::begin();
         gtar != ArteTable<GTAR>::end();
         ++gtar ) {
      diff = (*gtar)->x * (*gtar)->x + (*gtar)->y * (*gtar)->y;
      if (diff < 1 ) {
       xTARGET = (*gtar)->x;
       yTARGET = (*gtar)->y;
       zTARGET = (*gtar)->z;
      };
    };

//-----------------------------------------------------------------
//  loop over RICH and ECAL
//-----------------------------------------------------------------

    for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
         rseg != ArteTable<RSEG>::end();
         ++rseg ) {

    if ((*rseg)->cmp == 67108864 &&
        (*rseg)->ze >= 3.*sqrt(max(float(1.),(*rseg)->ye))){

    for( ArteTable<RCCL>::iterator rccl = ArteTable<RCCL>::begin();
         rccl != ArteTable<RCCL>::end();
         ++rccl ) {


         float dty, p, dx;
         dty = ((*rccl)->y - yTARGET)/((*rccl)->z - zTARGET)
                     - (*rseg)->tyf;
         dx  = (*rccl)->x - (*rseg)->txf * (*rccl)->z;

	 if (dx != 0.0) {
           p   = - 280. / dx;
         } else {
           p = 0.0;
         };
 



         if ((*rccl)->wid  >= 5 && fabs(dty)<0.002 &&
             fabs(p) > fabs(Cuts[4]) && 
             (*rccl)->esum < 1.3 * fabs(p) ){




//-----------------------------------------------------------------
//  fit RICH/ECAL track
//-----------------------------------------------------------------
    ArtePointer<RSEG> rRICH = *rseg;
    ArtePointer<RCCL> rECAL = *rccl;


  float pTrack[6];
  float chi2;
  float p0[3];
  float w0[2];

  p0[0] = xTARGET;
  p0[1] = yTARGET;
  p0[2] = zTARGET;
  w0[0] = 10.;
  w0[1] = 10.;

  gageFitEcalRich(rRICH, rECAL, p0, w0, pTrack, &chi2);


  if (chi2 < 20.){

    
    /*
   cout << " track after fit " << endl;
   cout << "  z    " << pTrack[0] << endl;
   cout << "  x    " << pTrack[1] << endl;
   cout << "  y    " << pTrack[2] << endl;
   cout << "  tx   " << pTrack[3] << endl;
   cout << "  ty   " << pTrack[4] << endl;
   cout << "  p    " << pTrack[5] << endl;
   cout << "  chi2 " << chi2 << endl;
    */

//-----------------------------------------------------------------
//  loop over subdetector segments
//-----------------------------------------------------------------

  for( ArteTable<RSEG>::iterator rsegAlign = ArteTable<RSEG>::begin();
       rsegAlign != ArteTable<RSEG>::end(); ++rsegAlign ) {

  if (((*rsegAlign)->fit == 0 || (*rsegAlign)->fit == 8) &&
       (*rsegAlign)->cmp & cmpAlign) {

  int goodAlignmentSegment = 1;

//-----------------------------------------------------------------
//  trace silicon track to SI06
//-----------------------------------------------------------------

  float z0;

  if (Cuts[4] > 0.0){
    z0 = 98.588;
  } else {
    z0 = zTARGET;
  };

  float pAlign[6];
  float pAlignTraced[6];

  pAlign[0] = (*rsegAlign)->zf;
  pAlign[1] = (*rsegAlign)->xf;
  pAlign[2] = (*rsegAlign)->yf;
  pAlign[3] = (*rsegAlign)->txf;
  pAlign[4] = (*rsegAlign)->tyf;
  pAlign[5] = 0.01;
  //  pAlign[5] = (*rsegAlign)->pf;


  gageTraceTRACK(z0 , pAlign, pAlignTraced);

//-----------------------------------------------------------------
//  redo ECAL RICH fit
//-----------------------------------------------------------------

  p0[0] = pAlignTraced[1];
  p0[1] = pAlignTraced[2];
  p0[2] = pAlignTraced[0];
  w0[0] = 10.;
  w0[1] = 10.;

  if (Cuts[4] > 0.0) gageFitEcalRich(rRICH, rECAL, p0, w0, 
                                     pTrack, &chi2);

//-----------------------------------------------------------------
//  cuts on Alignment segment segments
//-----------------------------------------------------------------

  /*
  cout << " vds segment segment " << endl;
  cout << " z   " << pAlignTraced[0] << endl;
  cout << " x   " << pAlignTraced[1] << endl;
  cout << " y   " << pAlignTraced[2] << endl;
  cout << " tx  " << pAlignTraced[3] << endl;
  cout << " ty  " << pAlignTraced[4] << endl;
  cout << " rho " << pAlignTraced[5] << endl;

  cout << " track traced to segment " << endl;
  cout << " z   " << p[0] << endl;
  cout << " x   " << p[1] << endl;
  cout << " y   " << p[2] << endl;
  cout << " tx  " << p[3] << endl;
  cout << " ty  " << p[4] << endl;
  cout << " rho " << p[5] << endl;
  */

  float dx, dy, dtx, dty;

  dx    = pTrack[1] - pAlignTraced[1];
  dy    = pTrack[2] - pAlignTraced[2];
  dtx   = pTrack[3] - pAlignTraced[3];
  dty   = pTrack[4] - pAlignTraced[4];

  /*
  cout << " cuts " << endl;
  cout << " dx  " << dx << endl;
  cout << " dy  " << dy << endl;
  cout << " dtx " << dtx << endl;
  cout << " dty " << dty << endl;
  */

  if (fabs(dx) >Cuts[0] || fabs(dy) >Cuts[1] || 
      fabs(dtx)>Cuts[2] || fabs(dty)>Cuts[3] )  goodAlignmentSegment = 0;

 //  cout << " goodAlignmentSegment  " << goodAlignmentSegment << endl;

//-----------------------------------------------------------------
//  fill histograms
//-----------------------------------------------------------------

    if (goodAlignmentSegment == 1 && gagePlotsVdsEcalRich == 1){

      rhp_hfill( AC_ID(COMP_CORR, LVL_REC, 106 ), dx , dy , 1 );
      rhp_hfill( AC_ID(COMP_CORR, LVL_REC, 107 ), dtx, dty, 1 );

    };

//-----------------------------------------------------------------
//  fill structure and pass to buffer
//-----------------------------------------------------------------

    if (goodAlignmentSegment == 1 && gageSelectVdsEcalRich == 1){

     GageSegmentPair SegmentPair;

     SegmentPair.key    = ArtePointer<EVHD>(1)->dbmajor;
     SegmentPair.z      = pAlignTraced[0];
     SegmentPair.p1[0]  = pAlignTraced[1];
     SegmentPair.p1[1]  = pAlignTraced[2];
     SegmentPair.p1[2]  = pAlignTraced[3];
     SegmentPair.p1[3]  = pAlignTraced[4];

     for (int i=0; i<10; i++) SegmentPair.V1[i] = (*rsegAlign)->cvf[i];

     SegmentPair.p2[0]  = pTrack[1];
     SegmentPair.p2[1]  = pTrack[2];
     SegmentPair.p2[2]  = pTrack[3];
     SegmentPair.p2[3]  = pTrack[4];

     for (int i=0; i<10; i++) SegmentPair.V2[i] = 0.0;


//-----------------------------------------------------------------
//  write to file or copy in rhp buffer
//-----------------------------------------------------------------

     if (gageFile != 0 ){

      fwrite(&SegmentPair, sizeof(GageSegmentPair), 1, gageFile);
      //      cout << " track written to file " << endl;

     } else {

      int size, currentsize;
      char *buf = rbf_obtain(AC_ID( COMP_CORR, LVL_REC, 1),
                             RBF_READ, &size, &currentsize); 
 
      int newsize = sizeof(GageSegmentPair) / sizeof(char);

      memcpy(buf, &SegmentPair, newsize);

      rbf_release(AC_ID( COMP_CORR, LVL_REC, 1), 
                  buf, newsize);

      //      cout << " track written to rhp buffer " << newsize << endl;

     };

    };
    };


  };
  };
 };
 };

};
};
};
