/***********************************************************************
* Author :    Siegmund Nowak   15/07/96
*
* Description: adds a mixture to GMAT and GEANT/JMATE and calculates the
*              medium parameters
*
* Called from: <user>
*
* cname  (A12) foresseen name of the mixture                    INPUT
* medflg IGMAT_FLAG (see description of table GMAT)             INPUT
* inmix  number of components in the mixture                    INPUT
* nammix(i) vector of component names (each A12)                INPUT
* volmix(i) vector of volume % of components (not normalized)   INPUT
***********************************************************************/
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/zgmed.hh"

extern "C" {
void crmixt_(char *name, int &medflg, int &inmix, char* nammix, float *volmix,
             int l_name, int l_nammix)   {

//     check that it is a mixture (it must be given with volume %)  
  if(inmix <= 1) {
     arteMsg << "crmixt is valid only for mixtures with input of volume %";
     arteMsg.warn();          return;
  }
  char cname[13];
//     look whether the mixture has been already booked
  if (l_name >12) l_name = 12;
  strncpy(cname,name,l_name);
  for (int i=0; i<l_name; i++) cname[i] = toupper(cname[i]);
  cname[l_name] = '\0'; 
  if(l_name < 12) strncat(cname,"            ",12-l_name);   
  for (int i=0; i<l_nammix; i++) nammix[i] = toupper(nammix[i]);

  zgmed ZGM;
  int ix = ZGM.Check_Gmat(cname);
  if(ix > 0) {  
    arteMsg << cname << " has been booked already - see entry " << ix;
    arteMsg.warn();          
    if(abs(inmix) > abs(ZGM.Getncomp(ix))) {
      arteMsg << "number of components in new bigger than in old " 
              << " define another material name ";
      arteMsg.error();
    } 
  }
  else {
    mmbook_(ArtePointer<GMAT>::getLink().getRef(), 1);
    ix = ArteTable<GMAT>::size();
  }

  ArtePointer<GMAT> gmat(ix); 
  gmat->flag = medflg;       
  gmat->nmix = inmix;
  strncpy(gmat->name,cname,12);
  unsigned int nim = abs(inmix);
  if(nim <= 1) gmat->gmix = 0;
  else gmat->gmix = ArteTable<GMIX>::size() + 1 ; 
  ZGM.Set_nmat(ix);

  for (int i = 0; i<inmix; i++)  ZGM.Fill_GMIX(&nammix[12*i],volmix[i]);
  ZGM.Create_Gmed();
}
}
