#include <iostream.h>
#include "arte/arte.hh"
#include "arte/Chrono.hh"   // A timer
#include "grover/grover.hh" // - class defs, f.prototypes and global variables

int GroverAddBest(GroverVertex* vtx,
		  GroverParticleList* particles,
		  int slowversion = 0)
{
  // Try to add a track to a (fitted!) vertex. Tracks labelled as 'used
  // in primary/secondary' will be ignored.
  // Each track used will be labelled with the index of the 
  // vertex. See manual for more info.
  // Better, but slower (4ms vs 0.8ms), algorithm used if third argument
  // set to 1. Use default first - the improvement is not huge for
  // the slower version.


  const float MAXC2D    =  36.0; // maximum particle-vtx chi2 distance
  const float MAXCHI2   =  50.0; // maximum chi2 of final fit
  bool slow = (slowversion != 0);// quick or clever? (clever=slower)

  GroverParticle* bestpar=0;

  if (slow) {
    // get best 3 particles
    float nice=1e9, nicer=1e9, nicest=1e9;
    GroverParticle* cands[3]={0};
    for (GroverParticleList::iterator ipar = particles->begin();
	 ipar != particles->end(); ++ipar) {
      if (ipar->UsedInSecondary() > 0 ||
	  ipar->UsedInPrimary() > 0) continue;
      float c2d = ipar->Chi2DistTo(vtx);
      if (c2d > MAXC2D) continue;
      if (c2d < nicest) {
	cands[2] = cands[1]; cands[1] = cands[0]; cands[0] = ipar;
	nice = nicer; nicer = nicest; nicest = c2d;
      }
      else if (c2d < nicer) {
	cands[2] = cands[1]; cands[1] = ipar;
	nice = nicer; nicer = c2d;
      }
      else if (c2d < nice) {
	cands [2] = ipar;
	nice = c2d;
      }
    }
    // any particles within sqrt(MAXC2D) sigma?
    if (cands[0] == 0) return 0; 
    // Find best fit
    float bestchi2 = MAXCHI2;
    float thischi2 = 0.0;
    for (int i = 0; i<3; i++) {
      if (cands[i] == 0) break;
      vtx->ResetParticles();
      vtx->Add(cands[i]);
      vtx->Fit(-1.0);
      thischi2 = vtx->Chi2();
      if (thischi2 < bestchi2) {
	bestchi2 = thischi2;
	bestpar  = cands[i];
      }
      vtx->particles.pop_back();
    }
  }


  // Quick version
  else {
    //vtx->ResetParticles();
    //vtx->Fit(-1.0);
    float nicest = 1e9;
    for (GroverParticleList::iterator ipar = particles->begin();
	 ipar != particles->end(); ++ipar) {
      if (ipar->UsedInSecondary() > 0 ||
	  ipar->UsedInPrimary() > 0) continue;
      float c2d = ipar->Chi2DistTo(vtx);
      if (c2d > MAXC2D) continue;
      if (c2d < nicest) {
	bestpar = ipar;
	nicest  = c2d;
      }
    }
  }



  // Add best particle if halfway reasonable
  if (bestpar != 0) {
    bestpar->SetUsedInSecondary(vtx->Index());
    vtx->Add(bestpar); vtx->Fit(-1.0);
  }


  return 0;
}
