#ifndef _RCSEGPOINT_HH_
#define _RCSEGPOINT_HH_
//-----------------------------------------------------------------------
// Created: 13.08.97,   Olya Igonkina

// MARPLE class of Points for Reconstruction which have link to RSEG bank

//-----------------------------------------------------------------------

#include <iostream.h>
#include "marple/RcPoint.hh"

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

/** class with link to RSEG bank.
    Class is inherited from RcPoint.
 */
class RcSegPoint : public RcPoint
{

public:
  ///                 default constructor
  inline RcSegPoint(); 
  ///                 constructor with link to RSEG bank    
  inline RcSegPoint( const RcPoint &rp, 
		     const ArtePointer <RSEG> rsegLink = 0 );
  /// constructor from RSEG bank entry , if EndPoint = 0 first point of RSEG is passed to RcSegPoint object, if any diagonal element of covariant matrix equal 0 it is set to infinity 
  RcSegPoint( const ArtePointer <RSEG> rsegLink,
	      int EndPoint = 0);

  ///                 copy constructor
  inline RcSegPoint( const RcSegPoint &rp ); 

  ///                 destructor
  inline ~RcSegPoint(); 

  ///                 return link to RSEG table
  inline ArtePointer <RSEG>  link() const; 

  ///                 set link to RSEG table
  inline void  setlink( const ArtePointer <RSEG> rsegLink = 0 );
  
  ///                 return relation with MTRA 
  const std::vector<ArtePointer<MTRA> > mtraList() const; 

  ///                 return relation with HITB 
  const std::vector<ArtePointer<HITB> > hitbList() const; 

  /// put RcSegPoint information to RSEG bank, the closest first/end point is changed by RcPoint
  void updateARTE( int & iErr ) const;

  /// update RSEG bank by RcSegPoint information
  void updateARTE( int & iErr,
		   int EndPoint ) const;

  ///                 assignment operator
  inline RcSegPoint & operator = (const RcSegPoint &rp);
  ///                 assignment operator
  inline RcSegPoint & operator = (const RcPoint &rp); 

private:
  ///                 ArtePointer to RSEG bank
  ArtePointer  <RSEG> fSegLink;
};

//                   ostream operator to print RcSegPoint information
ostream & operator << (ostream &, const RcSegPoint &);

#include "marple/RcSegPoint.icc"

#endif

