//--------------------------------------------------------------------
// Created: 02.09.97 by   Olya Igonkina
// see http://www-hera-b.desy.de/subgroup/software/arte/MARPLE/   

//   MARPLE matching of SVD segments and segments reconstructed 
//   in the Tracker area behind magnet(OT)
//   with estimation of momentum information 
//   by requirement of segment point to target

//--------------------------------------------------------------------
  

//--------------------------------------------------------------------

#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"

#include "arte/ArteMessage.hh"
#include "arte/RSEG_cmp.hh"

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chMatchSvdOtr = 0;
#endif

//definition of constants for SVD-OT matching ( see \Ref{RcMatchConst})
// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
// use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2, 
RcMatchConst rcSvdOtConst = { 
11110,     160.,   -30.,  800.,      0,          1,            1,
    1,      .8,      11.,      0.004,      .015,   1.e+10,   100000.,  
};

// MC RcMatchConst rcSvdOtConst = { 
//11110,     160.,   30.,  800.,      1,          1,            1,
//    1,      3.,      5.,      0.01,      .01,   1.e+10,   100.,  
//};

RcMatchConst rcSoftSvdOtConst = { 
11110,     160.,   -30.,  800.,      0,          1,            1,
    1,      3.,      35.,      0.015,      .05,   1.e+10,   1.e+10,  
};


extern "C"  
void FORTRAN_ROUTINE(matchsvdot)() 
{

# ifdef MARPLEPERFORMANCES
  if(chMatchSvdOtr == 0){
    ChronoManager CM;
    chMatchSvdOtr = CM.createChrono("Svd-Otr match");
  }
  chMatchSvdOtr->start();
# endif
  

  RcMatchBuilder< RcSegPoint, RcSegPoint> SvdOtMatch;

  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------
 if( MARPLE_SOFT )
   SvdOtMatch.setConst(rcSoftSvdOtConst);
  else
    SvdOtMatch.setConst(rcSvdOtConst);

  //--------------------------------------------------------------------
  // create vector of SVD segments
  //--------------------------------------------------------------------
  
  int iErr;
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
        irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if ( (*irseg)->fit != 0 ) continue;
      if( ((*irseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if( (*irseg)->cmp != Rsegc::vxd && 
	  (*irseg)->cmp != 65537) continue;
      if(  SvdOtMatch.ptToMatchConst()->geomAccept(HepDouble((*irseg)->ze)) )
	{ 
	  RcSegPoint * ptToPoint = new RcSegPoint( (*irseg) , 1 );
	  ptToPoint->prolongate( SvdOtMatch.ptToMatchConst()->z_plane,
				 iErr);
	  if( iErr )
	    {
	      delete ptToPoint;
	    }else{
	      SvdOtMatch.addPointLeft( *ptToPoint);
	    }
	  ptToPoint = 0;
	}
    }

  // no candidate found --- no reason go through OTR segments
  if( SvdOtMatch.vectorLeft().size() == 0 ) 
    {
# ifdef MARPLEPERFORMANCES
      chMatchSvdOtr->stop();
# endif
      return;
    }
  //--------------------------------------------------------------------
  // create vector of Outer Tracker segments 
  //--------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
        irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if ( (*irseg)->fit != 0 ) continue;
      if( ((*irseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if(  (((*irseg)->cmp & Rsegc::patt) || ((*irseg)->cmp & Rsegc::magt)) && 
	   SvdOtMatch.ptToMatchConst()->geomAccept(HepDouble((*irseg)->zf)) )
	{
	  iErr = 0;
	  RcSegPoint * ptToPoint = new RcSegPoint( (*irseg), 0 ); //first point
	  if( fabs(ptToPoint->pe() )> 2 || fabs(ptToPoint->pe() )<0.000001 )
	    {
	      // initial value from kick :
	      float overMomentum = 
		( ptToPoint->tx()*ptToPoint->z() - ptToPoint->x() ) / 288;
	      if( fabs(overMomentum) >  2 ) // Pt < 0.5 GeV
		{
		  iErr = 1;
		}else{
		  ptToPoint->setpe( overMomentum);
		  ptToPoint->estimatePEfromTAR( 5, iErr);
		}
	    }
	  if( fabs(ptToPoint->pe() )> 2 ) // must curls in the magnet
	    {
	      iErr = 1;
	    }

	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }
	  
	  ptToPoint->prolongate( SvdOtMatch.ptToMatchConst()->z_plane, iErr);
	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }

	  SvdOtMatch.addPointRight( *ptToPoint);
	  ptToPoint = 0;	      
	}
    }  
  
  //--------------------------------------------------------------------
  // create a list of matching points with chi2 
  //--------------------------------------------------------------------

  SvdOtMatch.doMatch();

  //--------------------------------------------------------------------
  // fill OTR estimated momentum to matchPoint
  //--------------------------------------------------------------------

  
  unsigned int nCombinations = SvdOtMatch.matchList().size();
  for( unsigned int iMatch = 0;  iMatch<nCombinations; ++iMatch)
    {
      RcPoint * otr = SvdOtMatch.rightOrigin(iMatch);
      SvdOtMatch.matchList()[iMatch]->setpe(  otr->pe() );
      SvdOtMatch.matchList()[iMatch]->setcov(5,5,otr->cov()(5,5));	  
    }      

    
  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------

  SvdOtMatch.stat();
  
  if( MARPLE_DEBUG >= 20)
    {  
      SvdOtMatch.fillDeltas(8213,8313);
      SvdOtMatch.fillMatch( 8113);
    }else if( MARPLE_DEBUG >= 10){
      SvdOtMatch.fillDeltas(8213);
    }
  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------
  if( MARPLE_MAKERSEG ) SvdOtMatch.saveRSEG(iErr);
  
  SvdOtMatch.saveRelation(iErr);
  if( iErr )
    { arteMsg << "MARPLE: some relations were not saved "; arteMsg.allp();}
  
  //    refit information
  //SvdOtMatch.refitLeftFound(       iErr);
  //SvdOtMatch.refitRightFound(      iErr);
  //    and update RSEG tables
  //SvdOtMatch.updateARTELeftFound(  iErr);
  //SvdOtMatch.updateARTERightFound( iErr);

  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  SvdOtMatch.deleteAll();  

# ifdef MARPLEPERFORMANCES
  chMatchSvdOtr->stop();
# endif
}

