/* Usage:
   
   exchange_i_j(f, (Arg1*)0, .., (ArgN*)0);

   where 2 <= N <= 5, 1 <= i < j <= N. (E.g. exchange_2_4(...)).
   F f - some function object which takes N arguments so that
   expression f(a1, .., aN)
   (a1, .., aN are of types Arg1, .., ArgN respectively)
   is valid.
   exchange_i_j(...) returns the function object (lets denote it by g) which 
   takes also N arguments but "i"th and "j"th arguments are exchanged:
   g(.. aj, .., ai, ..) = f(.. ai, .., aj, ..)

*/

// Author V.Balagura (balagura@mail.desy.de)   (09.04.98).

#ifndef _exchange_arguments_hh
#define _exchange_arguments_hh 1

/*---------------------- 2 args ----------------------- */
/*----------------- 2 args, 1 <-> 2 ------------------- */
template <class F, class A1, class A2>
class ExchangeArg2_1_2 {
	public:
	F function;
	ExchangeArg2_1_2(F f) : function(f) {}
	void operator()(A2 a2, A1 a1) { function(a1, a2); }
};
template <class F, class A1, class A2>
inline ExchangeArg2_1_2<F, A1, A2> exchange_1_2(F f, A1*, A2*) {
	return ExchangeArg2_1_2<F, A1, A2>(f);
}

/*---------------------- 3 args ----------------------- */
/*----------------- 3 args, 1 <-> 2 ------------------- */
template <class F, class A1, class A2, class A3>
class ExchangeArg3_1_2 {
	public:
	F function;
	ExchangeArg3_1_2(F f) : function(f) {}
	void operator()(A2 a2, A1 a1, A3 a3) { function(a1, a2, a3); }
};
template <class F, class A1, class A2, class A3>
inline ExchangeArg3_1_2<F, A1, A2, A3> exchange_1_2(F f, A1*, A2*, A3*) {
	return ExchangeArg3_1_2<F, A1, A2, A3>(f);
}
/*----------------- 3 args, 1 <-> 3 ------------------- */
template <class F, class A1, class A2, class A3>
class ExchangeArg3_1_3 {
	public:
	F function;
	ExchangeArg3_1_3(F f) : function(f) {}
	void operator()(A3 a3, A2 a2, A1 a1) { function(a1, a2, a3); }
};
template <class F, class A1, class A2, class A3>
inline ExchangeArg3_1_3<F, A1, A2, A3> exchange_1_3(F f, A1*, A2*, A3*) {
	return ExchangeArg3_1_3<F, A1, A2, A3>(f);
}
/*----------------- 3 args, 2 <-> 3 ------------------- */
template <class F, class A1, class A2, class A3>
class ExchangeArg3_2_3 {
	public:
	F function;
	ExchangeArg3_2_3(F f) : function(f) {}
	void operator()(A1 a1, A3 a3, A2 a2) { function(a1, a2, a3); }
};
template <class F, class A1, class A2, class A3>
inline ExchangeArg3_2_3<F, A1, A2, A3> exchange_2_3(F f, A1*, A2*, A3*) {
	return ExchangeArg3_2_3<F, A1, A2, A3>(f);
}
/*---------------------- 4 args ----------------------- */
/*----------------- 4 args, 1 <-> 2 ------------------- */
template <class F, class A1, class A2, class A3, class A4>
class ExchangeArg4_1_2 {
	public:
	F function;
	ExchangeArg4_1_2(F f) : function(f) {}
	void operator()(A2 a2, A1 a1, A3 a3, A4 a4) { function(a1, a2, a3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline ExchangeArg4_1_2<F, A1, A2, A3, A4>
exchange_1_2(F f, A1*, A2*, A3*, A4*) {
	return ExchangeArg4_1_2<F, A1, A2, A3, A4>(f);
}
/*----------------- 4 args, 1 <-> 3 ------------------- */
template <class F, class A1, class A2, class A3, class A4>
class ExchangeArg4_1_3 {
	public:
	F function;
	ExchangeArg4_1_3(F f) : function(f) {}
	void operator()(A3 a3, A2 a2, A1 a1, A4 a4) { function(a1, a2, a3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline ExchangeArg4_1_3<F, A1, A2, A3, A4>
exchange_1_3(F f, A1*, A2*, A3*, A4*) {
	return ExchangeArg4_1_3<F, A1, A2, A3, A4>(f);
}
/*----------------- 4 args, 2 <-> 3 ------------------- */
template <class F, class A1, class A2, class A3, class A4>
class ExchangeArg4_2_3 {
	public:
	F function;
	ExchangeArg4_2_3(F f) : function(f) {}
	void operator()(A1 a1, A3 a3, A2 a2, A4 a4) { function(a1, a2, a3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline ExchangeArg4_2_3<F, A1, A2, A3, A4>
exchange_2_3(F f, A1*, A2*, A3*, A4*) {
	return ExchangeArg4_2_3<F, A1, A2, A3, A4>(f);
}
/*----------------- 4 args, 1 <-> 4 ------------------- */
template <class F, class A1, class A2, class A3, class A4>
class ExchangeArg4_1_4 {
	public:
	F function;
	ExchangeArg4_1_4(F f) : function(f) {}
	void operator()(A4 a4, A2 a2, A3 a3, A1 a1) { function(a1, a2, a3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline ExchangeArg4_1_4<F, A1, A2, A3, A4>
exchange_1_4(F f, A1*, A2*, A3*, A4*) {
	return ExchangeArg4_1_4<F, A1, A2, A3, A4>(f);
}
/*----------------- 4 args, 2 <-> 4 ------------------- */
template <class F, class A1, class A2, class A3, class A4>
class ExchangeArg4_2_4 {
	public:
	F function;
	ExchangeArg4_2_4(F f) : function(f) {}
	void operator()(A1 a1, A4 a4, A3 a3, A2 a2) { function(a1, a2, a3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline ExchangeArg4_2_4<F, A1, A2, A3, A4>
exchange_2_4(F f, A1*, A2*, A3*, A4*) {
	return ExchangeArg4_2_4<F, A1, A2, A3, A4>(f);
}
/*----------------- 4 args, 3 <-> 4 ------------------- */
template <class F, class A1, class A2, class A3, class A4>
class ExchangeArg4_3_4 {
	public:
	F function;
	ExchangeArg4_3_4(F f) : function(f) {}
	void operator()(A1 a1, A2 a2, A4 a4, A3 a3) { function(a1, a2, a3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline ExchangeArg4_3_4<F, A1, A2, A3, A4>
exchange_3_4(F f, A1*, A2*, A3*, A4*) {
	return ExchangeArg4_3_4<F, A1, A2, A3, A4>(f);
}
/*---------------------- 5 args ----------------------- */
/*----------------- 5 args, 1 <-> 2 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_1_2 {
	public:
	F function;
	ExchangeArg5_1_2(F f) : function(f) {}
	void operator()(A2 a2, A1 a1, A3 a3, A4 a4, A5 a5) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_1_2<F, A1, A2, A3, A4, A5>
exchange_1_2(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_1_2<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 1 <-> 3 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_1_3 {
	public:
	F function;
	ExchangeArg5_1_3(F f) : function(f) {}
	void operator()(A3 a3, A2 a2, A1 a1, A4 a4, A5 a5) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_1_3<F, A1, A2, A3, A4, A5>
exchange_1_3(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_1_3<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 1 <-> 4 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_1_4 {
	public:
	F function;
	ExchangeArg5_1_4(F f) : function(f) {}
	void operator()(A4 a4, A2 a2, A3 a3, A1 a1, A5 a5) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_1_4<F, A1, A2, A3, A4, A5>
exchange_1_4(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_1_4<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 1 <-> 5 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_1_5 {
	public:
	F function;
	ExchangeArg5_1_5(F f) : function(f) {}
	void operator()(A5 a5, A2 a2, A3 a3, A4 a4, A1 a1) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_1_5<F, A1, A2, A3, A4, A5>
exchange_1_5(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_1_5<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 2 <-> 3 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_2_3 {
	public:
	F function;
	ExchangeArg5_2_3(F f) : function(f) {}
	void operator()(A1 a1, A3 a3, A2 a2, A4 a4, A5 a5) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_2_3<F, A1, A2, A3, A4, A5>
exchange_2_3(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_2_3<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 2 <-> 4 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_2_4 {
	public:
	F function;
	ExchangeArg5_2_4(F f) : function(f) {}
	void operator()(A1 a1, A4 a4, A3 a3, A2 a2, A5 a5) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_2_4<F, A1, A2, A3, A4, A5>
exchange_2_4(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_2_4<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 2 <-> 5 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_2_5 {
	public:
	F function;
	ExchangeArg5_2_5(F f) : function(f) {}
	void operator()(A1 a1, A5 a5, A3 a3, A4 a4, A2 a2) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_2_5<F, A1, A2, A3, A4, A5>
exchange_2_5(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_2_5<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 3 <-> 4 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_3_4 {
	public:
	F function;
	ExchangeArg5_3_4(F f) : function(f) {}
	void operator()(A1 a1, A2 a2, A4 a4, A3 a3, A5 a5) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_3_4<F, A1, A2, A3, A4, A5>
exchange_3_4(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_3_4<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 3 <-> 5 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_3_5 {
	public:
	F function;
	ExchangeArg5_3_5(F f) : function(f) {}
	void operator()(A1 a1, A2 a2, A5 a5, A4 a4, A3 a3) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_3_5<F, A1, A2, A3, A4, A5>
exchange_3_5(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_3_5<F, A1, A2, A3, A4, A5>(f);
}
/*----------------- 5 args, 4 <-> 5 ------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class ExchangeArg5_4_5 {
	public:
	F function;
	ExchangeArg5_4_5(F f) : function(f) {}
	void operator()(A1 a1, A2 a2, A3 a3, A5 a5, A4 a4) {
		function(a1, a2, a3, a4, a5);
	}
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline ExchangeArg5_4_5<F, A1, A2, A3, A4, A5>
exchange_4_5(F f, A1*, A2*, A3*, A4*, A5*) {
	return ExchangeArg5_4_5<F, A1, A2, A3, A4, A5>(f);
}

#endif
