// 
//  Author:  Rainer Mankel                     Created:  24-Aug-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//
// Modified:   
// - moved many routines to State.C
//                                     14-Aug-00 Wouter Hulsbergen

#undef TRANSCD

#include <iostream>
#include <math.h>
#include "rang/StateP.hh"
#include "rang/zfpar.hh"
#include "rang/rkmain.hh"
#include "rang/utfeld.hh"
#include "rang/rktranscd.hh"
#include "arte/ArteMessage.hh"

//rangerStateP::rangerStateP() : rangerState(5) {} 

int rangerStateP::predict(const double& zDestination,
			 const double& qPExp,rangerState*& s,
			 HepMatrix*& F) const {

  int newState     = (s == 0);  // denote if memory allocations exist
  int newTransport = (F == 0);   //   .    .    .        .         .

#ifdef FULL
  cout << "rangerStateP::predict: zDestination=" << zDestination << endl;

  cout << "rangerStateP::predict: targetState=" << s
       << " newState= " <<  newState << endl;
#endif

  if (newState) {
    s = new rangerStateP();
    if (!s) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }
  if (! F) {
    F = new HepMatrix(5,5);
    if (!F) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }

#ifdef FULL
  cout << "rangerStateP::predict: after allocation: s->vec()=" << s->vec() 
       << endl << "       *F=" << *F << endl;
#endif

  double deltaZ = fabs(zDestination - zState);

  if ( (zDestination>zfpar_.zmaxMag && zState>zfpar_.zmaxMag)
       || (zDestination<zfpar_.zminMag && zState<zfpar_.zminMag)
       || deltaZ < 0.001) {
    s->setZ( zDestination );      // linear transport
    s->setX( x() + (zDestination - zState) * tx() );
    s->setY( y() + (zDestination - zState) * ty() );
    s->setTx( tx() );
    s->setTy( ty() );
    s->setQp( qP() );
    s->setQpEst( qPEst() );

    for (int i=1; i<=5; i++) {
      for (int j=1; j<=5; j++) {
	(*F)(i,j) = 0.;
      }
    }
    (*F)(1,1) = 1;
    (*F)(2,2) = 1;
    (*F)(3,3) = 1;
    (*F)(4,4) = 1;
    (*F)(5,5) = 1;
    (*F)(1,3) = zDestination - zState;
    (*F)(2,4) = zDestination - zState;

    s->cov() = c.similarity(*F);

    if (fabs(s->x()) < 700);
    else {
      arteMsg << "StateP::predict: produced garbage (lin), vec="
	      << s->vec() << endl;
      cout <<  "StateP::predict: produced garbage (lin), vec="
	      << s->vec() << endl;
      //double cc = cin.get();
      return -10;
    } 

  }
  else {
#ifdef TRANSCD
    double fstep = fabs(zDestination - zState);
    if (fstep<=0) fstep = 1;
    double pIn[5];
    pIn[0] = x();
    pIn[1] = y();
    pIn[2] = tx();
    pIn[3] = ty();
    pIn[4] = qPExp;

    double pOut[5];
    int ierror;

    double cOut[25];

    if ( abs(pIn[0]) < 700 &&   // check validity of rkmain input
	 abs(pIn[1]) < 700 &&
	 abs(pIn[2]) < 10   &&
	 abs(pIn[3]) < 10   &&
	 abs(pIn[4]) < 100 );
    else {
      cout << "StateP::predict: invalid pIn="
	   << pIn[0] << " "
	   << pIn[1] << " "
	   << pIn[2] << " "
	   << pIn[3] << " "
	   << pIn[4] 
	   << " transporting from " << zState
	   << " to " << zDestination << endl;
      return -10;
    }

    double cIn[25];
    double rkd[25];
    for (int i=0; i<=4; i++) {
      for (int j=0; j<=4; j++) {
    	cIn[5*i+j] = c(i+1,j+1);
     }
    }

    //rkmain_(icovar,zout,fstep,pin,prec,pOut,rkd,ierror);
    //rktrans_(zState,pin,zDestination,pOut,rkd,ierror);
    double zout = zDestination;
    rktranscd_(zState,pIn,cIn,zout,pOut,cOut,rkd,ierror);

    if (ierror == 0) {
      for (int i=1; i<=5; i++) {
	for (int j=1; j<=5; j++) {
	  (*F)(i,j) = rkd[5*(j-1)+i-1];
	}
      }

      s->setZ() = zDestination;
      s->setX() = pOut[0] + (*F)(1,5) * (qP() - qPExp);
      s->setY() = pOut[1] + (*F)(2,5) * (qP() - qPExp);
      s->setTx() = pOut[2] + (*F)(3,5) * (qP() - qPExp);
      s->setTy() = pOut[3] + (*F)(4,5) * (qP() - qPExp);
      s->setQp() = qP();  // only valid in case momentum is unchanged
      s->setQpEst() = qPEst();

      for (int i=1; i<=5; i++) {
	for (int j=1; j<=5; j++) {
	  s->cov()(i,j) = cOut[5*(j-1)+i-1];
	}
      }

      if (abs(s->x()) < 700);
      else {
	cout << "StateP::predict: produced garbage (mag), vec="
	     << s->vec() << endl;
	cout << " The input was " 
	     << pIn[0] << " "
	     << pIn[1] << " "
	     << pIn[2] << " "
	     << pIn[3] << " "
	     << pIn[4] 
	     << " transporting from " << zState
	     << " to " << zDestination << endl;
	cout << " The output was " 
	     << pOut[0] << " "
	     << pOut[1] << " "
	     << pOut[2] << " "
	     << pOut[3] << endl;
	//double cc = cin.get();
	return -10;
      } 
    }
    else {
      cout << "state::predict: rkmain returns ierror=" << ierror << endl;
      return -1;
    }


#else
    double zout = zDestination;
    double fstep = fabs(zDestination - zState);
    if (fstep<=0) fstep = 1;
    double pin[6];
    pin[0] = zState;
    pin[1] = x();
    pin[2] = y();
    pin[3] = tx();
    pin[4] = ty();
    pin[5] = qPExp;
    double prec[4];
    double dzTrunc = fabs(zDestination - zState);
    if (dzTrunc<1) dzTrunc = 1;

    prec[0] = 0.001 / dzTrunc;
    prec[1] = 0.001 / dzTrunc;
    prec[2] = 0.0001 / dzTrunc;
    prec[3] = 0.0001 / dzTrunc;

    int icovar = 1;
    //int icovar = 2;
    double pout[6];
    double rkd[25];
    int ierror;
    rkmain_(icovar,zout,fstep,pin,prec,pout,rkd,ierror);

    if (ierror == 0) {
      if (pout[0] != zDestination) 
	cout << "stateP::predict: zDest changed from "
	     << zDestination << " to " << pout[0] << endl;
      for (int i=1; i<=5; i++) {
	for (int j=1; j<=5; j++) {
	  (*F)(i,j) = rkd[5*(j-1)+i-1];
	}
      }
      s->setZ( zDestination );
      s->setX( pout[1] + (*F)(1,5) * (qP() - qPExp) );
      s->setY( pout[2] + (*F)(2,5) * (qP() - qPExp) );
      s->setTx( pout[3] + (*F)(3,5) * (qP() - qPExp) );
      s->setTy( pout[4] + (*F)(4,5) * (qP() - qPExp) );
      s->setQp( qP() );  // only valid in case momentum is unchanged
      s->setQpEst( qPEst() );
    }
    else {
      cout << "state::predict: rkmain returns ierror=" << ierror << endl;
      return -1;
    }
    s->cov() = c.similarity(*F);
#endif



  }


#ifdef FULL
  cout << "rangerStateP::predict: after calculation, *F=" << *F << endl;

  cout << "rangerStateP::predict: the transported vector is"
       << s->vec() << endl;
#endif

  //s->cov() = c.similarity(*F);

#ifdef FULL
  cout << "rangerStateP::predict: the transported cov is"
       << s->cov() << endl;
#endif

  if (1) {                // everything OK
    return 0;
  }
  else {
    cout << " This should not happen!!!" << endl;
    if (newState) delete s;
    if (newTransport) delete F;
    return -1000;
  }
}




int rangerStateP::predictOld(const double& zDestination,
			 const double& qPExp,rangerState*& s,
			 HepMatrix*& F) {

  int newState     = (s == 0);  // denote if memory allocations exist
  int newTransport = (F == 0);   //   .    .    .        .         .

#ifdef FULL
  cout << "rangerStateP::predictOld: zDestination=" << zDestination << endl;

  cout << "rangerStateP::predictOld: targetState=" << s
       << " newState= " <<  newState << endl;
#endif

  if (newState) {
    s = new rangerStateP();
    if (!s) {
      cout << "rangerState::predictOld: allocation error" << endl; 
      exit(1);
    }
    F = new HepMatrix(5,5);
    if (!F) {
      cout << "rangerState::predictOld: allocation error" << endl; 
      exit(1);
    }
  }

#ifdef FULL
  cout << "rangerStateP::predictOld: after allocation: s->vec()=" << s->vec() 
       << endl << "       *F=" << *F << endl;
#endif

  double deltaZ = fabs(zDestination - zState);

  if ( (zDestination>zfpar_.zmaxMag && zState>zfpar_.zmaxMag)
       || (zDestination<zfpar_.zminMag && zState<zfpar_.zminMag)
       || deltaZ < 0.001) {
    s->setZ( zDestination );      // linear transport
    s->setX( x() + (zDestination - zState) * tx() );
    s->setY( y() + (zDestination - zState) * ty() );
    s->setTx( tx() );
    s->setTy( ty() );
    s->setQp( qP() );
    s->setQpEst( qPEst() );

    for (int i=1; i<=5; i++) {
      for (int j=1; j<=5; j++) {
	(*F)(i,j) = 0.;
      }
    }
    (*F)(1,1) = 1;
    (*F)(2,2) = 1;
    (*F)(3,3) = 1;
    (*F)(4,4) = 1;
    (*F)(5,5) = 1;
    (*F)(1,3) = zDestination - zState;
    (*F)(2,4) = zDestination - zState;
  }
  else {
    double zout = zDestination;
    double fstep = fabs(zDestination - zState);
    if (fstep<=0) fstep = 1;
    double pin[6];
    pin[0] = zState;
    pin[1] = x();
    pin[2] = y();
    pin[3] = tx();
    pin[4] = ty();
    pin[5] = qPExp;
    double prec[4];
    double dzTrunc = fabs(zDestination - zState);
    if (dzTrunc<1) dzTrunc = 1;

    prec[0] = 0.001 / dzTrunc;
    prec[1] = 0.001 / dzTrunc;
    prec[2] = 0.0001 / dzTrunc;
    prec[3] = 0.0001 / dzTrunc;

    int icovar = 1;
    //int icovar = 2;
    double pOut[6];
    double rkd[25];
    int ierror;

    if ( fabs(pin[1]) < 700 &&   // check validity of rkmain input
	 fabs(pin[2]) < 700 &&
	 fabs(pin[3]) < 10   &&
	 fabs(pin[4]) < 10   &&
	 fabs(pin[5]) < 100 );
    else {
      cout << "StateP::predictOld: invalid pin="
	   << pin[1] << " "
	   << pin[2] << " "
	   << pin[3] << " "
	   << pin[4] << " "
	   << pin[5] << endl;
      return -2;
    }


    rkmain_(icovar,zout,fstep,pin,prec,pOut,rkd,ierror);

    if (ierror == 0) {
      if (pOut[0] != zDestination) 
	cout << "state::predictOld: zDest changed from "
	     << zDestination << " to " << pOut[0] << endl;
      for (int i=1; i<=5; i++) {
	for (int j=1; j<=5; j++) {
	  (*F)(i,j) = rkd[5*(j-1)+i-1];
	}
      }
      s->setZ( zDestination );
      s->setX( pOut[1] + (*F)(1,5) * (qP() - qPExp) );
      s->setY( pOut[2] + (*F)(2,5) * (qP() - qPExp) );
      s->setTx( pOut[3] + (*F)(3,5) * (qP() - qPExp) );
      s->setTy( pOut[4] + (*F)(4,5) * (qP() - qPExp) );
      s->setQp( qP() );  // only valid in case momentum is unchanged
      s->setQpEst( qPEst() );
    }
    else {
      cout << "state::predictOld: rkmain returns ierror=" << ierror << endl;
      return -1;
    }
  }


#ifdef FULL
  cout << "rangerStateP::predictOld: after calculation, *F=" << *F << endl;

  cout << "rangerStateP::predictOld: the transported vector is"
       << s->vec() << endl;
#endif

  s->cov() = c.similarity(*F);

#ifdef FULL
  cout << "rangerStateP::predictOld: the transported cov is"
       << s->cov() << endl;
#endif

  if (1) {                // everything OK
    return 0;
  }
  else {
    cout << " This should not happen!!!" << endl;
    if (newState) delete s;
    if (newTransport) delete F;
    return -1000;
  }
}
