// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Slight modification in inclusion of header files
//                                      14-Aug-00 Wouter Hulsbergen
//

// class rangerHit
// node class for HITB-based measurement


#ifndef rangerHit_HH
#define rangerHit_HH

#ifndef rangerHitBase_HH
#include "rang/HitBase.hh"
#endif
#ifndef rangerVolume_HH
#include "rang/Volume.hh"
#endif

class rangerHit : public rangerHitBase {

public:

  rangerHit();
  rangerHit(ArtePointer<RHIT> rhit,rangerFitMode fm);
  rangerHit(ArtePointer<HITB>   hb,rangerFitMode fm);
  virtual ~rangerHit();   // virtual destructor
  
  inline float ms() const { return sPath*vol->ms(); }
  inline float msPlain() const { ArtePointer<GEDE> ge = hitb->gede;
                                 return ge->ms; }
  float dz();           // thickness in z
  int plane();

  virtual int dim() const=0;       // dimension of measurement
  inline const ArtePointer<HITB> getHitb() const { return hitb;} 
                                      // Accessor for hitb
  inline int hasHitb() { return 1;}
  virtual void storeRhit(ArtePointer<RHIT> rhit) ;
  void mergeRhit(ArtePointer<RHIT> rhit,float weight)
    { state->mergeRhit(rhit,weight); };

private:
  ArtePointer<HITB> hitb;    // Arte pointer to HITB table
  rangerVolume* vol;     // pointer to volume
  float sPath;
};

#endif
