// 
//  Author:  Rainer Mankel                     Created:  19-May-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

#include "rang/DomWin.hh"
#include "rang/Domain.hh"

// rgdomwin_
// Fortran interface to obtain array of domain window pointers associated
// with a domain. The domain pointer is taken as input parameter.
// The domain pointers are written into a fixed-length array defined by
// the calling routine. Length of and pointer to this array are taken
// as parameters. Previous contents of array are overwritten. The number of
// written domain window pointers is returned as last parameter.

extern "C" {
  void rgdomwin_(rangerDomain*& dm,        // domain pointer (input)
		 const int& maxWin,        // max array length (input)
		 rangerDomWin** windowVec, // array pointer
		 int& nWin) {              // number of windows (output)
    
    if (! dm) return;

    nWin = 0;
    for (vector <rangerDomWin*>::iterator wit=dm->win().begin();
	 wit != dm->win().end(); ++wit) {
      if (nWin < maxWin) {
	windowVec[nWin] = *wit;
	++nWin;
      }
    }
	
    return;
  }
}
	  
