// 
//  Author:  Rainer Mankel                     Created:  11-Jun-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

#include "rang/DomLay.hh"
#include "rang/ProxyLay.hh"
#include "arte/arte.hh"

// rghlcrpr_
// Same as rghlcr, except that proxy windows are taken into account.
// For a given domain layer, particle arrow and tracking mode, give 
// array of HITB hits suitable for track.
// The HITB row numbers are stored into a fixed-length array defined
// in the calling routine. The number of hits in array is reset to zero
// before filling. 




extern "C" {
  void rghlcr_(rangerDomLay* const & dl,
	       const int& tMode,
	       const double& z,const double& x,const double& y,
	       const double& tx,const double& ty,
	       const int& maxHits,
	       ArtePointer<HITB>* hitVec,
	       int& nHits);
}


extern "C" {
  void rghlcrpr_(rangerDomLay*& dl,    // domain layer
		 const int& tMode,     // tracking mode, can be
		                       //      1 - 2D x
                                       //      2 - 2D y
                                       //      3 - 3D straight
                                       //      4 - 3D mag
		 const double& z,      // z of prediction
		 const double& x,      // x
		 const double& y,      // y
		 const double& tx,     // tan theta-x
		 const double& ty,     // tan theta-y
		 const int& maxHits,   // max space in array
		 ArtePointer<HITB>* hitVec, // pointer to hit array for output
		 int& nHits) {         // number of hits in array

    nHits = 0;

    // get hitlist for dl
    rghlcr_(dl,tMode,z,x,y,tx,ty,maxHits,hitVec,nHits);

    // loop over proxy layers
    for (vector<rangerProxyLay*>::iterator pli=dl->proxylay().begin();
	 pli != dl->proxylay().end(); ++pli) {
      if ((*pli)->inside((float) y)) {
	int nHit1 = 0;
	rghlcr_((*pli)->tolay(),tMode,z,x,y,tx,ty,
		maxHits-nHits,&(hitVec[nHits]),nHit1);
	//cout << "rghlcrpr: " << nHit1 << " proxy hits added" 
	//     << " to " << nHits << " existing hits" << endl;
	nHits += nHit1;
      }
    }
  }
}
