*-- Author :    Siegmund Nowak / Rainer Mankel   15/01/95

      subroutine tfthel(ndim, x_in, dz, x_out, f, istat)

*     **** transport for helices
      implicit none
      integer ndim, istat
      double precision x_in(5), x_out(5), f(5,5)
      double precision hc, curv, dz, z1ovr, z2ovr, deltax
      double precision deltay, sqim1, sqz1r, sqz2r
#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "arte/Arte.ff"
#include "auto/GEO.ff"

*     **** check ndim
      if (ndim.ne.5) then
       print *,' () tfthel: ndim=', ndim
       istat = -1
       go to 999
      endif

*     **** fixup for very small curvature
      hc = .00029975*ffeld_h(1)
      curv = hc * x_in(5)
      if (abs(curv).gt.0.00001) then

*      **** predictions
       z1ovr = x_in(3) / dsqrt(1.d0 + x_in(3)**2)
       z2ovr = z1ovr - dz * curv

*      **** check if radius large enough to reach plane
       if (dabs(z1ovr).lt.1.d0.and.dabs(z2ovr).lt.1.d0) then
        istat = 0
       else
        print '((a),f10.3,(a),f10.5,(a),f10.5)',
     +         ' tfthel: transport impossible, dz=',dz,'1/r=',
     +         curv,' tx=',x_in(3)
        istat = -1
        go to 999
       endif

       sqz1r = dsqrt(1.d0 - z1ovr**2)
       sqz2r = dsqrt(1.d0 - z2ovr**2)
       sqim1  = 1.d0 / dsqrt(1.d0 + x_in(3)**2)

       deltax = - ( sqz1r - sqz2r ) / curv
       x_out(1) = x_in(1) + deltax

       deltay = - x_in(4) * sqim1 * (dasin(z2ovr) - dasin(z1ovr))
     +                                                    / curv
       x_out(2) = x_in(2) + deltay

       x_out(3) = z2ovr / sqz2r

       x_out(4) = x_in(4) * dsqrt(1. + x_out(3)**2) * sqim1
**       x_out(4) = x_in(4) / (dsqrt(1. + x_out(3)**2) * sqim1)

       x_out(5) = x_in(5)

*      **** derivatives

       f(1,1) = 1.d0
       f(1,2) = 0.d0
       f(1,3) = - (x_out(3) - x_in(3)) * sqim1**3 / curv
       f(1,4) = 0.d0
       f(1,5) = (x_out(3) * dz - deltax) / x_in(5)

       f(2,1) = 0.d0
       f(2,2) = 1.d0
       f(2,3) = -(x_in(4)/curv)
     +        * ( -x_in(3) * sqim1**3 * (dasin(z2ovr) - dasin(z1ovr))
     +              + sqim1**4 * (1.d0/sqz2r - 1.d0/sqz1r) )
       f(2,4) = - sqim1 * (dasin(z2ovr) - dasin(z1ovr)) / curv
       f(2,5) = ( -deltay + x_in(4) * sqim1 * dz / sqz2r )
     +                                                    / x_in(5)

       f(3,1) = 0.d0
       f(3,2) = 0.d0
       f(3,3) = sqim1**3 / sqz2r**3
       f(3,4) = 0.d0
       f(3,5) = - hc * dz / sqz2r**3

       f(4,1) = 0.d0
       f(4,2) = 0.d0
       f(4,3) = ( -x_in(3) * (1.-z2ovr**2) + z2ovr * sqim1 )
     +          * x_in(4) * sqim1**3 / sqz2r**3
       f(4,4) = dsqrt(1. + x_out(3)**2) * sqim1
       f(4,5) = - hc * x_in(4) * sqim1 * z2ovr * dz / sqz2r**3

       f(5,1) = 0.d0
       f(5,2) = 0.d0
       f(5,3) = 0.d0
       f(5,4) = 0.d0
       f(5,5) = 1.d0
      else
*      **** predictions for quasi-linear transport
       sqim1  = 1.d0 / dsqrt(1.d0 + x_in(3)**2)

       x_out(1) = x_in(1) + dz * x_in(3)
       x_out(2) = x_in(2) + dz * x_in(4)
       x_out(3) = x_in(3)
       x_out(4) = x_in(4)
       x_out(5) = x_in(5)

*      **** derivatives
       f(1,1) = 1.d0
       f(1,2) = 0.d0
       f(1,3) = dz
       f(1,4) = 0.d0
       f(1,5) = -.5 * hc * dz**2 / sqim1**3

       f(2,1) = 0.d0
       f(2,2) = 1.d0
       f(2,3) = 0.d0
       f(2,4) = dz
       f(2,5) = -0.5 * hc * x_in(3) * x_in(4) / sqim1 * dz**2

       f(3,1) = 0.d0
       f(3,2) = 0.d0
       f(3,3) = 1.
       f(3,4) = 0.d0
       f(3,5) = - hc * dz / sqim1**3

       f(4,1) = 0.d0
       f(4,2) = 0.d0
       f(4,3) = 0.
       f(4,4) = 1.d0
       f(4,5) = - hc * x_in(3) * dsqrt(1.d0 + x_in(3)**2) * x_in(4) * dz

       f(5,1) = 0.d0
       f(5,2) = 0.d0
       f(5,3) = 0.d0
       f(5,4) = 0.d0
       f(5,5) = 1.d0
      endif

 999  end


