
// *****************************************************************************
//
// source:    rcClassJPSIMM.C
//
// type:      source code
//
// created:   20 Jun 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcClassJPSIMM
//
// called by: rcClassify
//
// purpose:   check for J/Psi -> mu+mu- event
//
// hints:     do NOT set flagdst here!
//            muonjpsi:     >= 0 0x1
//            SelectJPSIMM: >  0 0x2
//
// changes:
// 20 Jun 2000 (AG) creation
//  6 Jul 2000 (AG) added FORTRAN function muonjpsi_()
//  7 Jul 2000 (AG) renamed muonjpsi_() to rcmuonjpsi_()
//  6 Oct 2000 (AG) added rcSelectJPSIMM
//
// *****************************************************************************

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// globals
float massJPSIMM = 0;

// prototypes
extern "C" float rcmuonjpsi_();
int rcSelectJPSIMM();

//
// function code
//
int
rcClassJPSIMM()
{
  ARTE_mark( "rcClassJPSIMM 0", 0 );

  // inits
  int rc = 0;

  //
  // invoke J/Psi -> mu+mu- finder
  //
  ARTE_mark( "rcmuonjpsi_", 0 );
  massJPSIMM = rcmuonjpsi_();
  if( massJPSIMM >= 0 ) rc |= 0x1;

  //
  // invoke rcSelectJPSIMM()
  //
  ARTE_mark( "rcSelectJPSIMM" , 0 );
  int flag = rcSelectJPSIMM();
  if( flag ) rc |= 0x2;

  ARTE_mark( "rcClassJPSIMM 1", 0 );
  return rc;
}
