// *****************************************************************************
//
// source:    attach muon segments to rtra with vds-(rich or otr)
//
// type:      source code
//
// created:   07 Aug 2000
//	
// author:   D. Dujmic/K. Reeves
//
// changes:
//  08 Aug 2000  (DD) Remove a debug printout
//
// *****************************************************************************
#include "arte/arte.hh"
#include "arte/RSEG_cmp.h"
#include "vector.h"
#include "reco/rcXkinema.h"
#include "reco/rcXsegment.h"
#include "reco/rcXmessage.h"
#include "CLHEP/Hist/HBookHistogram.h"


//#define DEBUG_HIS

// 
//  create relation of RTRA's with muon segments
//
int rcMuonRTRA(int verb) {

  // -- tune sigmas, chi2 ?
  static float isig_dtx = 1./.008;
  static float isig_dty = 1./.010;
  static float isig_dx  = 1./5.0;
  static float isig_dy  = 1./8.1;
  static float max_chi2 = 30;
#ifdef DEBUG_HIS
  static HBookHistogram hchi2("hchi2",100,0,100);
  static HBookHistogram hdx("hdx",50,-100,100);
  static HBookHistogram hdy("hdy",50,-100,100);
  static HBookHistogram hdtx("hdtx",50,-.1,.1);
  static HBookHistogram hdty("hdty",50,-.1,.1);
#endif

  static vector<ArtePointer<RSEG> >           muon_list;
  static vector<ArtePointer<RSEG> >::iterator mit;
  if (muon_list.size()) muon_list.clear();

  // -- find muon segments in RSEG
  for (ArteTable<RSEG>::iterator irseg=ArteTable<RSEG>::begin();
       irseg!=ArteTable<RSEG>::end(); ++irseg) {
    if ((*irseg)->cmp==Rsegc::muon) muon_list.push_back(*irseg);
  }
  if (verb) cout<<"muonRTRA: found "<<muon_list.size()<<" muons"<<endl;
  if (!muon_list.size()) return RCX_OK;


  // -- loop over charged tracks in RTRA
  int total=0;
  for (ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
       irtra!=ArteTable<RTRA>::end(); ++irtra) {

    // -- only charged tracks
    if ((*irtra)->fit%100) continue;
    if (((*irtra)->cer<.040) && ((*irtra)->hou+(*irtra)->hin<15)) continue;
    if ((*irtra)->hsi<5) continue;
    if (fabs((*irtra)->pf)>0.1) continue; // 10 GeV

    // -- swim behind magnet (can be approximated with a kick)
    double pin[]={(*irtra)->zf, (*irtra)->xf, (*irtra)->yf, 
		    (*irtra)->txf, (*irtra)->tyf, (*irtra)->pf};
    double pou[6];
    kinema::swim(pin,pou,1300);

    // -- check for muon
    for (mit=muon_list.begin(); mit!=muon_list.end(); ++mit) {
      float dz  = (*mit)->zf - pou[0];
      float dx  = (*mit)->xf - pou[1] - dz*pou[3];
      float dy  = (*mit)->yf - pou[2] - dz*pou[4];
      float dtx = (*mit)->txf - pou[3];
      float dty = (*mit)->tyf - pou[4];
#ifdef DEBUG_HIS
      if(fabs(dy)<30) hdx.accumulate(dx);
      if(fabs(dx)<20) hdy.accumulate(dy);
      if(fabs(dty)<0.030) hdtx.accumulate(dtx);
      if(fabs(dtx)<0.020) hdty.accumulate(dty);
#endif
      dx *=isig_dx;
      dy *=isig_dy;
      dtx*=isig_dtx;
      dty*=isig_dty;

      // -- check chi2
      float chi2=dx*dx + dy*dy + dtx*dtx + dty*dty;
#ifdef DEBUG_HIS
      hchi2.accumulate(chi2);
#endif
      if (chi2>max_chi2) continue;
     
      // -- create relation
      createRelation((*mit), (*irtra), &RSEG::rtra);
      (*irtra)->hmu=((*mit)->nhit>0) ? (*mit)->nhit : 1;
      (*irtra)->cmuon=(*mit).index();
      if (verb) cout<<"muonRTRA: create relation rtra="<<(*irtra)<<" rseg="<<(*mit)<<" hit="<<(*irtra)->hmu<<endl;
      total++;
    }
  }
  if (verb) cout<<"muonRTRA: appended "<<total<<" muons"<<endl;

  return RCX_OK;
}
