
// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) wire constraint, mass histogram 
//
// *****************************************************************************

#include "reco/rcX.h"
#include "reco/rcXkinema.h"
#include "farm/farm.hh"
#include "rhp.h"
#include "dq/DQ.h"

rcX *rcKs1=0;
rcX *rcKs2=0;

int copyRTRItoRTRA(int algorithm);

int Ks__Piplus_Piminus_MC();
int Ks__Piplus_Piminus_RECO();
int maximizeRTRA();

//
//
//
int
Ks()
{
  if (ArteTable<MTRA>::size()) {
    //Ks__Piplus_Piminus_MC();
  }
  Ks__Piplus_Piminus_RECO();
  return 0;
}

//
//
//
int
Ks__Piplus_Piminus_MC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>>>  Ks1 <<<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcKs1) {
    cout<<"initializing Ks"<<endl;
    rcKs1=new rcX("Ks", "Ks -> Pi+Pi-", verb);
    decay ks;
    ks.fit           = 3;   // = long decay
    ks.book_tables   = 1;   // 1/0
    ks.rlevel        = 2;   // 0=full, 1=semi, 2=mtra
    ks.verb          = verb;// printout level
    rcKs1->setMother(ks);   // set
    vertexAttrib vks;
    vks.rlevel           = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vks.topp             = kinema::findTopp(310); // pointer to mother
    vks.momentum_flag    = 1;     // momentum in fit?
    vks.max_vertex_chi2  = 10.0;  // limit for vertex acceptance 
    vks.verb             = verb;  // printout level
    vks.min_dz_from_wire = 5;     // minimum decay distance (cm)
    vks.min_mass         = .4;    // minimum mass
    vks.max_mass         = .6;    // maximum mass
    rcKs1->setVertex(vks);        // set

    attribute pi1;
    pi1.topp   =kinema::findTopp(-211);
    pi1.lund   =-211;
    pi1.mother =310;
    pi1.mass   =.139567;
    pi1.minP   =-200;
    pi1.maxP   =0;
    pi1.minHRI =1;
    pi1.minLRPI=.001;
    pi1.useVdsOnce   = 1;
    pi1.useMainOnce  = 1;
    rcKs1->addDaughter(pi1);

    attribute pi2;
    pi2.topp   =kinema::findTopp(211);
    pi2.lund   =211;
    pi2.mother =310;
    pi2.mass   =.139567;
    pi2.minP   =0;
    pi2.maxP   =200;
    pi2.minHRI =1;
    pi2.minLRPI=.001;
    pi2.useVdsOnce   = 1;
    pi2.useMainOnce  = 1;
    rcKs1->addDaughter(pi2);
  
    if (verb) rcKs1->printDaughters();
  }
  cout<<"calling rc"<<endl;
  rcKs1->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>>>  Ks1 <<<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}

//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
///////////////////////////  Ks reconstruction   /////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////

int
Ks__Piplus_Piminus_RECO()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  rec Ks2 <<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcKs2) {
    cout<<"initializing Ks"<<endl;
    rcKs2=new rcX("rec_Ks", "Ks -> Pi+Pi", verb);
    decay ks;
    ks.rlevel        = 0;    // 0=full, 1=semi, 2=mtra
    ks.fit           = 103;    // = long decay 3 + flag 100 = 103
    ks.book_tables   = 1;    // 1/0
    ks.root_output   = 1;    // make root file or not
    ks.verb          = verb; // printout level
    rcKs2->setMother(ks);    // set
    vertexAttrib vks;
    vks.rlevel             = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vks.topp               = kinema::findTopp(310); // pointer to mother
    vks.max_vertex_chi2    = 10.0;  // limit for vertex acceptance 
    vks.verb               = verb;  // printout level
    vks.min_dz_from_wire   = 5;     // minimum decay distance (cm)
    vks.max_drho_from_wire = .5;    // check is decay points toward wire 
    vks.min_mass           = .4;    // min mass
    vks.max_mass           = .6;    // max mass
    vks.allowed_v0_tracks  = 1;     // pions not from v0
    rcKs2->setVertex(vks);          // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(-211);
    pi1.mass         = pi1.topp->mass;
    pi1.minP         = -200;
    pi1.maxP         = 0;
    pi1.minPt        = .15;
    pi1.minLRPI      = 1e-4;
    pi1.maxLRE       = 1e-5;
    pi1.minHRI       = 1;
    pi1.minHSI       = 6;
    pi1.maxCandidate = 15;
    pi1.useVdsOnce   = 1;
    pi1.useMainOnce  = 1;
    rcKs2->addDaughter(pi1);

    attribute pi2;
    pi2.topp         = kinema::findTopp(211);
    pi2.mass         = pi2.topp->mass;
    pi2.minP         = 0;
    pi2.maxP         = 200;
    pi2.minPt        = .15;
    pi2.minLRPI      = 1e-4;
    pi2.maxLRE       = 1e-5;
    pi2.minHRI       = 1;
    pi2.minHSI       = 6;
    pi2.maxCandidate = 15;
    pi2.useVdsOnce   = 1;
    pi2.useMainOnce  = 1;
    rcKs2->addDaughter(pi2);

    if (verb) rcKs2->printDaughters();
  }
  rcKs2->rc();
  if (verb) rcKs2->printTracks();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  rec Ks2 <<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}




//
//
//
int
selectKs()
{
  ARTE_mark( "rcX_Ks 0", 0 );
  static int old_total=0;
  int ret=0;
  copyRTRItoRTRA(0); // use ringsearch
  rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1030 ), 0., 1.); // new event
  ArteTable<RTRA>::iterator ks1=ArteTable<RTRA>::end();
  Ks();
  ArteTable<RTRA>::iterator ks2=ArteTable<RTRA>::end();
  if (rcKs2->total>old_total) { 
    old_total=rcKs2->total;
    rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1030 ), 1., 1.); // Ks found
    // -- fill mass
    for (ArteTable<RTRA>::iterator iks=ks1; iks!=ks2; ++iks) {
      ArtePointer<RVER> rver=(*iks)->ever;
      float mass=rver->mass;
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1031 ), mass, 1.);
    }
    ret=1; 
  }
  ARTE_mark( "rcX_Ks 1", 0 );
  //cout<<"selectKs ret="<<ret<<endl;
  return ret;
}

//
//
//
void
stopKs()
{
  if (rcKs1) delete rcKs1;
  if (rcKs2) delete rcKs2;
}
