
// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 01 Aug 2000 (DD) correct ecal,mtra from rseg
//
// *****************************************************************************

#include "reco/rcXsegment.h"

ArteTable<RSEG>::iterator segment::ivds =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::iitr =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::iotr =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::itc  =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::irich=ArteTable<RSEG>::end();
ArteTable<RCCL>::iterator segment::iecal=ArteTable<RCCL>::end();
ArteTable<RSEG>::iterator segment::imuon=ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::ivds_rich =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::ivds_ecal =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::irich_ecal=ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::ivds_otr  =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::ivds_itr  =ArteTable<RSEG>::end();
ArteTable<MTRA>::iterator segment::imtra  =ArteTable<MTRA>::end();
ArteTable<RCCL>::iterator segment::ibremse=ArteTable<RCCL>::end();
int                       segment::track_components=0;
int                       segment::covij[5][5]={{0,1,3,6,10},{1,2,4,7,11},{3,4,5,8,12},{6,7,8,9,13},{10,11,12,13,14}};


void segment::findSegments(ArtePointer<RTRA> t) {
  resetSegments();
  // -- find RSEG segment
  for (ArteRelationIterator<RSEG> irseg=t->rseg.begin();
       irseg!=t->rseg.end(); ++irseg) {
    resolveCmp(ArteTableIterator<RSEG>(*irseg));
  }

  // -- find ECAL cluster
  if (t->rccl.begin() != t->rccl.end()) { 
    iecal=ArteTableIterator<RCCL>(*t->rccl.begin()); 
    track_components=track_components|Rsegc::ecal; 
  }

  // -- find MTRA
  if (t->mtra.begin() != t->mtra.end()) { 
    imtra=ArteTableIterator<MTRA>(*t->mtra.begin()); 
  }
}


void segment::findSegments(ArtePointer<RSEG> mt) {
  resetSegments();
  // -- find RSEG
  resolveCmp(ArteTableIterator<RSEG>(mt));
  if (mt->fit!=7) return;
  for (ArteRelationIterator<RSEG>  jrseg=mt->rend.begin(); 
       jrseg!=mt->rend.end(); ++jrseg) {
    resolveCmp(ArteTableIterator<RSEG>(*jrseg));
  }
  // -- find ECAL
  if (mt->rccl.begin()!=mt->rccl.end()) {
    iecal=ArteTableIterator<RCCL>(*mt->rccl.begin());
    track_components=track_components|Rsegc::ecal;
  }
  // -- find MTRA
  if (mt->mtra.begin()!=mt->mtra.end()) {
    imtra=ArteTableIterator<MTRA>(*mt->mtra.begin()); 
  }
}


void segment::resolveCmp(ArteTable<RSEG>::iterator irseg) {
  if ((*irseg)->cmp&Rsegc::bitsign) return;

  // -- pure segments
  if ((*irseg)->fit!=7) {
    switch ((*irseg)->cmp) {
    case (Rsegc::vxd)                          :
    case (Rsegc::vxd|Rsegc::bit16)             : 
      ivds = irseg; break;

    case (Rsegc::msg|Rsegc::sil|Rsegc::trit)              : 
    case (Rsegc::msg|Rsegc::trit)                         :
    case (Rsegc::msg|Rsegc::patt|Rsegc::trit)             : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::trit|Rsegc::magt) : 
      itc = irseg; 
    case (Rsegc::msg|Rsegc::patt)                         :
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt)             : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::sil)              :
    case (Rsegc::msg|Rsegc::patt|Rsegc::sil|Rsegc::magt)  :
      iitr = irseg; break;

    case (Rsegc::wch|Rsegc::trit)                         :
    case (Rsegc::wch|Rsegc::patt|Rsegc::trit)             :
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::trit) : 
      itc = irseg;
    case (Rsegc::wch|Rsegc::magt)                         :
    case (Rsegc::wch|Rsegc::patt)                         : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt)             : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::sil)  : 
      iotr = irseg; break;

    case (Rsegc::msg|Rsegc::wch|Rsegc::trit)                       :  
      itc = irseg;
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt)                       :   
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt)           :
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::sil):
      iotr = irseg; iitr = irseg; break;

    case (Rsegc::rich)                         : irich = irseg; break;

    case (Rsegc::muon)                         : imuon = irseg; break;
    default: 
      cout<<"segment::findSegments: ERROR segment not found "<<hex<<(*irseg)->cmp<<dec<<endl;
      break;
    }
  }
  else {
    // -- matched segment
    switch ((*irseg)->cmp) {

    case (Rsegc::rich|Rsegc::ecal)             : irich_ecal = irseg; break;

    case (Rsegc::rich|Rsegc::vxd)              : 
    case (Rsegc::rich|Rsegc::vxd|Rsegc::bit16) : ivds_rich  = irseg; break;

    case (Rsegc::ecal|Rsegc::vxd)              :
    case (Rsegc::ecal|Rsegc::vxd|Rsegc::bit16) : ivds_ecal  = irseg; break;

    case (Rsegc::wch|Rsegc::patt|Rsegc::vxd)                           : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::vxd|Rsegc::bit16)              : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::vxd)               : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::vxd|Rsegc::bit16)  : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::sil|Rsegc::vxd)    : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::sil|Rsegc::vxd|Rsegc::bit16):
      ivds_otr = irseg;  break;

    case (Rsegc::msg|Rsegc::patt|Rsegc::vxd)                           : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::vxd|Rsegc::bit16)              : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt|Rsegc::vxd)               : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt|Rsegc::vxd|Rsegc::bit16)  : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt|Rsegc::sil|Rsegc::vxd)               : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt|Rsegc::sil|Rsegc::vxd|Rsegc::bit16)  : 
      ivds_itr = irseg;  break;

    case (Rsegc::msg|Rsegc::wch|Rsegc::sil|Rsegc::patt|Rsegc::magt)    :
    case (Rsegc::msg|Rsegc::wch|Rsegc::sil|Rsegc::patt|Rsegc::magt|Rsegc::vxd|Rsegc::bit16):
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::vxd)    :
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::vxd|Rsegc::bit16):
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::vxd)                :                           
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::vxd|Rsegc::bit16)   :                           
      ivds_itr = irseg;  ivds_otr = irseg;  break;

    default: 
      cout<<"segment::findSegments: ERROR segment not found "<<hex<<(*irseg)->cmp<<dec<<endl;
      break;
    }
  }
  track_components=(track_components|(*irseg)->cmp);
}


void segment::printSegments(ArtePointer<RTRA> t) {
  findSegments(t);
  cout<<t;
  if (vdsf()||t->hsi)      cout<<" vds ";
  if (itrf()||t->hin)      cout<<" itr ";
  if (otrf()||t->hou)      cout<<" otr ";
  if (tcf())               cout<<" tc ";
  if (richf()||t->cer>0)   cout<<" rich ";
  if (ecalf()||t->hca)     cout<<" ecal ";
  if (muonf()||t->hmu)     cout<<" muon ";
  if (vds_otrf())          cout<<" vds_otr ";
  if (vds_richf())         cout<<" vds_rich ";
  if (vds_ecalf())         cout<<" vds_ecal ";
  cout<<endl;
}

void segment::printSegments(ArtePointer<RSEG> s) {
  findSegments(s);
  cout<<s;
  printSegments();
}

void segment::printSegments() {
  cout<<" "<<hex<<track_components<<dec<<":";
  if (vdsf())  cout<<" vds ";
  if (itrf())  cout<<" itr ";
  if (otrf())  cout<<" otr ";
  if (tcf())   cout<<" tc ";
  if (richf()) cout<<" rich ";
  if (ecalf()) cout<<" ecal ";
  if (muonf()) cout<<" muon ";
  if (vds_otrf())  cout<<" vds_otr ";
  if (vds_richf()) cout<<" vds_rich ";
  if (vds_ecalf()) cout<<" vds_ecal ";
  cout<<endl;
}

void segment::resetSegments() {
  iecal=ibremse=ArteTable<RCCL>::end();
  irich=ivds=iotr=imuon=itc=iitr=ArteTable<RSEG>::end();
  ivds_rich=ivds_ecal=irich_ecal=ivds_otr=ivds_itr=ArteTable<RSEG>::end();
  imtra=ArteTable<MTRA>::end();
  track_components=0;
}


int segment::TrackDiff(ArtePointer<RTRA> t1, ArtePointer<RTRA> t2) {
  if (t1==t2) return 1;

  int ret=0;
  ArtePointer<RVER> v1;
  int               v1f=0;
  if (t1->fit%100==2 || t1->fit%100==3)  { v1=t1->ever; v1f=1; }
  ArtePointer<RVER> v2;
  int               v2f=0;
  if (t2->fit%100==2 || t2->fit%100==3)  { v2=t2->ever; v2f=1; }

  if (v1f && !v2f) {
    for (ArteRelationIterator<RREL> irrel=v1->rrel.begin();
	 irrel!=v1->rrel.end(); ++irrel) {
      if ((ret=checkTrackDiff((*irrel)->rtra,t2))) return ret;
    }
  }

  else if (!v1f && v2f) {
    for (ArteRelationIterator<RREL> irrel=v2->rrel.begin();
	 irrel!=v2->rrel.end(); ++irrel) {
      if ((ret=checkTrackDiff(t1,(*irrel)->rtra))) return ret;
    }
  }

  else if (v1f && v2f) {
    for (ArteRelationIterator<RREL> irrel=v1->rrel.begin();
	 irrel!=v1->rrel.end(); ++irrel) {
      for (ArteRelationIterator<RREL> jrrel=v2->rrel.begin();
	   jrrel!=v2->rrel.end(); ++jrrel) {
	if ((ret=checkTrackDiff((*irrel)->rtra,(*jrrel)->rtra))) return ret;
      }
    }
  }

  else ret=checkTrackDiff(t1,t2);

  return ret;
}
 


int segment::checkTrackDiff(ArtePointer<RTRA> t1, ArtePointer<RTRA> t2) {
  int ret=0;
 
  // -- mini/dst - only rtra is available
  if (t1->csvd==t2->csvd&&t1->csvd>0)    ret=(ret|Rsegc::vxd);
  if (t1->cmain==t2->cmain&&t1->cmain>0) ret=(ret|Rsegc::wch);
  if (t1->crich==t2->crich&&t1->crich>0) ret=(ret|Rsegc::rich);
  if (t1->cecal==t2->cecal&&t1->cecal>0) ret=(ret|Rsegc::ecal);
  if (t1->cmuon==t2->cmuon&&t1->cmuon>0) ret=(ret|Rsegc::muon);
 
  if (ret) return ret;
 
  // -- if rseg table is available 
  findSegments(t1);
  int vds1 = (vdsf())  ? (*ivds).index() : 0;
  int otr1 = (otrf())  ? (*iotr).index() : 0;
  int rich1= (richf()) ? (*irich).index(): 0;
  int ecal1= (ecalf()) ? (*iecal).index(): 0;
  int muon1= (muonf()) ? (*imuon).index(): 0;
  
  findSegments(t2);
  int vds2 = (vdsf())  ? (*ivds).index() : 0;
  int otr2 = (otrf())  ? (*iotr).index() : 0;
  int rich2= (richf()) ? (*irich).index(): 0;
  int ecal2= (ecalf()) ? (*iecal).index(): 0;
  int muon2= (muonf()) ? (*imuon).index(): 0;

  if (vds1  && vds2  && vds1==vds2)     ret=(ret|Rsegc::vxd);
  if (otr1  && otr2  && otr1==otr2)     ret=(ret|Rsegc::wch);
  if (rich1 && rich2 && rich1==rich2)   ret=(ret|Rsegc::rich);
  if (ecal1 && ecal2 && ecal1==ecal2)   ret=(ret|Rsegc::ecal);
  if (muon1 && muon2 && muon1==muon2)   ret=(ret|Rsegc::muon);
 
  return ret;
}


int segment::TrackDiff(ArtePointer<RSEG> s1, ArtePointer<RTRA> t2) {
  int ret=0;

  // -- if rseg table is available 
  findSegments(s1);
  int vds1 = (vdsf())  ? (*ivds).index() : 0;
  int otr1 = (otrf())  ? (*iotr).index() : 0;
  int rich1= (richf()) ? (*irich).index(): 0;
  int ecal1= (ecalf()) ? (*iecal).index(): 0;
  int muon1= (muonf()) ? (*imuon).index(): 0;
  
  findSegments(t2);
  int vds2 = (vdsf())  ? (*ivds).index() : 0;
  int otr2 = (otrf())  ? (*iotr).index() : 0;
  int rich2= (richf()) ? (*irich).index(): 0;
  int ecal2= (ecalf()) ? (*iecal).index(): 0;
  int muon2= (muonf()) ? (*imuon).index(): 0;

  if (vds1  && vds2  && vds1==vds2)     ret=(ret|Rsegc::vxd);
  if (otr1  && otr2  && otr1==otr2)     ret=(ret|Rsegc::wch);
  if (rich1 && rich2 && rich1==rich2)   ret=(ret|Rsegc::rich);
  if (ecal1 && ecal2 && ecal1==ecal2)   ret=(ret|Rsegc::ecal);
  if (muon1 && muon2 && muon1==muon2)   ret=(ret|Rsegc::muon);

  return ret;
}


void  segment::killRTRA(int fitflag, int itopp) {
  ArtePointer<TOPP> topp;
  if (itopp) topp=ArtePointer<TOPP>(itopp);
  for (ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
       irtra!=ArteTable<RTRA>::end(); ++irtra) {
    if ((*irtra)->fit%100!=fitflag)    continue;
    if (itopp && (*irtra)->topp!=topp) continue;
    (*irtra)->topp=9999;
  }
}

