!-----------------------------------------------------------------------
! cerenkov angle calculation for track-photon pair
! M. Staric, jan-2000
!
!-----------------------------------------------------------------------

      SUBROUTINE CALC_THCER(RF,SIG_F,TH_CER,FI_CER,SIG)

      REAL RF(3), SIG_F
      REAL TH_CER, FI_CER
      REAL SIG

#include "riter/RIC_TRIC.fi"
#include "riter/RIC_SFM.fi"

      LOGICAL IN_NARROW_WIN, IN_WIDE_WIN, IN_NARROW_WIN2, IN_WIDE_WIN2
      LOGICAL IS_PBS_ONWAY, IS_EBS_ONWAY
      LOGICAL OK
      REAL DIRF(3), DIRCH(3), R1(3), DIRF1(3)

      SIG=0.
! which detector
      K0=KJE_HIT(RF(2))
! coarse limitation
      X=RF(1)-TRC(1,K0)
      Y=RF(2)-TRC(2,K0)
      Z=RF(3)-TRC(3,K0)
      RADIJ2=X**2+Y**2+Z**2
      IF(RADIJ2.GT.RMAX2) RETURN
! approximate cerenkov angle**2
      RADIJ2=RADIJ2/(R_MI(K0)/2.)**2
      IF(.NOT.IN_WIDE_WIN2(RADIJ2)) RETURN
! accurate cerenkov angle
      ID=ID_SPH(K0)
      IF(ID.GE.0) THEN
         CALL CERENKOV_KOT(TR0,TDIR,RF,SF_RC(1,ID),SF_RMI(ID),
     &        DIRF,DIRCH,OK)         
      ELSE
         CALL CERENKOV_KOT(TR0,TDIR,RF,RCMI(1,K0),R_MI(K0),
     &        DIRF,DIRCH,OK)         
      ENDIF
      IF(.NOT.OK) RETURN
! was photon emitted by this track (emit it from entrance to RICH)
      CALL SFM_HIT(TR,DIRF,K0,PATH,OK)
      IF(.NOT.OK) RETURN
      CALL NEW_POINT(TR,DIRF,PATH,R1)
      IF(K0.EQ.0) THEN
         OK=KJE_T.EQ.0.OR.R1(2).GE.0
      ELSE
         OK=KJE_T.EQ.1.OR.R1(2).LE.0
      ENDIF
      IF(.NOT.OK) RETURN
! screening of beam shroud (too restrictive: some photons are lost!!)
! before reflection
      IF(IS_PBS_ONWAY(TR,DIRF,PATH1)) THEN
         IF(PATH1.LT.PATH) RETURN
      ENDIF
! after reflection
      CALL SFM_REFLECT(R1,DIRF,K0,DIRF1,AA)
      IF(IS_PBS_ONWAY(R1,DIRF1,PATH)) RETURN
! cerenkov theta, phi
      TH_CER=SQRT(DIRCH(1)**2+DIRCH(2)**2)
      IF(TH_CER.GT.0) THEN
         FI_CER=ATAN2(DIRCH(2),DIRCH(1))
      ELSE
         FI_CER=0.
      ENDIF
! corrections for misaligned mirrors, if data is given
      CALL GET_CORRECT_FLAG(OK)
      IF(OK) THEN
         CALL SFM_HIT(TR0,DIRF,K0,PATH,OK)
         IF(.NOT.OK) RETURN
         CALL NEW_POINT(TR0,DIRF,PATH,R1)
         KK=KJE_HIT(R1(2))
         IF(KK.EQ.K0) THEN
            CALL WHICH_SFM(R1,K0,ISFM)
         ELSE
            ISFM=ID_SPH(K0)
         ENDIF
         CALL SFM_REFLECT(R1,DIRF,K0,DIRF1,AA)
         CALL PLM_HIT(R1,DIRF1,K0,PATH,OK)
         IF(.NOT.OK) RETURN
         CALL NEW_POINT(R1,DIRF1,PATH,R1)
         CALL WHICH_PLM(R1,K0,IPLM)
         CALL CORRECT_THC(ISFM,IPLM,TH_CER,FI_CER)
      ENDIF
! set angular sigma and sign flag
      CFI2=COS(FI_CER)**2
      SIG=SQRT(SIG_F**2+SIG_T2(1)*CFI2+SIG_T2(2)*(1.-CFI2))
      IF(.NOT.IN_NARROW_WIN(TH_CER)) SIG=-SIG
  
      RETURN
      END



      SUBROUTINE CERENKOV_KOT(R0,DIR,RH,RC,R_MI,DIRF,DIRCH,OK)

! calculation of cerenkov angle from photon hit on detector and part. track
! resi_sin: linear approximation

      REAL R0(3)                ! point of photon emission (mid point on track)
      REAL DIR(3)               ! particle direction (unit vector)
      REAL RH(3)                ! photon hit on detector
      REAL RC(3), R_MI          ! center of curvature of mirror

      REAL DIRF(3)              ! result (photon direction vector in HERA-B) if
      REAL DIRCH(3)             ! result (photon direction vector in DIR) if
      LOGICAL OK                ! OK = .true.

      REAL R1(3), R2(3), RR(3), R1N, R2N, RRN
      REAL A, B, ALFA, TH, FI
      REAL C1, C2, S_ALFA

      DO I=1,3
         R1(I)=R0(I)-RC(I)
         R2(I)=RH(I)-RC(I)
      ENDDO
      R1N=SQRT(R1(1)**2+R1(2)**2+R1(3)**2)	
      R2N=SQRT(R2(1)**2+R2(2)**2+R2(3)**2)	
      A=R1N/R_MI
      B=R2N/R1N
      ALFA=(R1(1)*R2(1)+R1(2)*R2(2)+R1(3)*R2(3))/R1N/R2N
      ALFA=MAX(MIN(ALFA,1.),-1.)
      ALFA=ACOS(ALFA)
      CALL RESI_SIN(A,B,ALFA,TH,FI,OK)
      IF(.NOT.OK) RETURN
      S_ALFA=SIN(ALFA)
      C1=R_MI/R1N*SIN(ALFA-FI)/S_ALFA
      C2=R_MI/R2N*SIN(FI)/S_ALFA
      DO I=1,3
         RR(I)=(C1-1.)*R1(I)+C2*R2(I)
      ENDDO
      RRN=SQRT(RR(1)**2+RR(2)**2+RR(3)**2)	
      DO I=1,3
         DIRF(I)=RR(I)/RRN
      ENDDO
      CALL TRANSFORM_FROM_FIXED(DIR,DIRF,DIRCH)
      
      RETURN
      END


      SUBROUTINE RESI_SIN(A,B,C,X,Y,OK)
      
! find solution of equations:	sin(x) = a*sin(x+y)
!				sin(x+y)=b*sin(c+x-y)
! approximative solution (second order)

      REAL A,B,C                ! coeficients
      REAL X,Y                  ! result
      LOGICAL OK                ! if all OK
      
      ABB=1.-2*A*B+B
      IF(ABB.EQ.0) THEN
         OK=.FALSE.
         RETURN
      ENDIF

      X=A*B*C/ABB
      Y=(1.-A)*B*C/ABB

      SX=SIN(X)
      CX=COS(X)
      SXY=SIN(X+Y)
      CXY=COS(X+Y)
      SCXY=SIN(C+X-Y)
      CCXY=COS(C+X-Y)
      
      A11=CX-A*CXY
      A12=-A*CXY
      B1=A*SXY-SX
      A21=CXY-B*CCXY
      A22=CXY+B*CCXY
      B2=B*SCXY-SXY
         
      D=A11*A22-A12*A21
      IF(D.EQ.0) THEN
         OK=.FALSE.
         RETURN
      ENDIF
      
      X=X+(B1*A22-B2*A12)/D
      Y=Y+(B2*A11-B1*A21)/D

      OK=.TRUE.
      RETURN
      END
	  	

      SUBROUTINE CORRECT_THC(ID_SPH,ID_PLM,TH_CER,FI_CER)

      INTEGER ID_SPH, ID_PLM
      REAL TH_CER, FI_CER

      INTEGER LUN
      CHARACTER*(*) FILE_NAME
      INTEGER IO

      CHARACTER*80 TT, SS
      REAL DX_SPH(0:79) /80*0./, DY_SPH(0:79) /80*0./
      REAL DX_PLM(0:35) /36*0./, DY_PLM(0:35) /36*0./
      LOGICAL NO_CORR /.TRUE./
      SAVE DX_SPH, DY_SPH, DX_PLM, DY_PLM, NO_CORR
      LOGICAL FLAG

      IF(NO_CORR) RETURN
      IF(ID_SPH.LT.0) RETURN
      IF(ID_PLM.LT.0) RETURN
      X0=DX_SPH(ID_SPH)+DX_PLM(ID_PLM)
      Y0=DY_SPH(ID_SPH)+DY_PLM(ID_PLM)
      TH_CER=TH_CER-(X0*COS(FI_CER)+Y0*SIN(FI_CER))
      RETURN

      ENTRY GET_CORRECT_FLAG(FLAG)

      FLAG=.NOT.NO_CORR
      RETURN

      ENTRY READ_MIRR_CORR(LUN,FILE_NAME,IO)

cc--- this one should be changed in the future 
cc--- as it can't run on the farm ... 
cc--- (it's anyway not called in this version of riter)

      DO I=0,79
         DX_SPH(I)=0.
         DY_SPH(I)=0.
      ENDDO
      DO I=0,35
         DX_PLM(I)=0.
         DY_PLM(I)=0.
      ENDDO
      IF(FILE_NAME.EQ.' ') RETURN
      OPEN(LUN,FILE=FILE_NAME,STATUS='OLD',IOSTAT=IO)
      IF(IO.NE.0) THEN
         PRINT*,'can''t open file: ',FILE_NAME
         RETURN
      ENDIF
      NMI=0
 1    CONTINUE
      READ(LUN,'(A)',ERR=1,END=99) TT
      CALL STRIP_OFF(TT,L)
      IF(L.EQ.0.OR.TT(1:1).EQ.'#') GOTO 1
      CALL SUB_STRING(TT,SS,L)
      IF(SS(1:L).EQ.'S') THEN
         READ(TT,*,IOSTAT=IO) ID,X,Y,EX,EY
         IF(IO.NE.0.OR.ID.LT.0.OR.ID.GT.79) THEN
            PRINT*,'file corrupted: ',SS(1:L),'|',TT
            GOTO 99
         ENDIF
         DX_SPH(ID)=X/1000.
         DY_SPH(ID)=Y/1000.
         NMI=NMI+1
      ELSE IF(SS(1:L).EQ.'P') THEN
         READ(TT,*,IOSTAT=IO) ID,X,Y,EX,EY
         IF(IO.NE.0.OR.ID.LT.0.OR.ID.GT.35) THEN
            PRINT*,'file corrupted: ',SS(1:L),'|',TT
            GOTO 99
         ENDIF
         DX_PLM(ID)=X/1000.
         DY_PLM(ID)=Y/1000.
         NMI=NMI+1
      ENDIF
      GOTO 1

 99   CONTINUE
      CLOSE(LUN)
      PRINT*,NMI,' corrections for mirrors read'
      NO_CORR=NMI.EQ.0

      RETURN
      END



