#include "L3_RFBand.hh"
#include "L3_Track.hh"
#include <math.h>

ostream& operator<<(ostream& cout,const L3_RFBand& a){
  cout << "x: " << a.xmin() << ".." << a.xmax() << " (" << a.dx() << ')' << endl;
  cout << "y: " << a.ymin() << ".." << a.ymax() << " (" << a.dy() << ')' << endl;
  cout << "z: " << a.zmin() << ".." << a.zmax() << " (" << a.dz() << ')' << endl;
  cout << "slope/zstop : " << a.slope() << '/' << a.zstop() << endl;
  return cout;
}

#ifdef L3_ARTE_FRAME
L3_RFBand::L3_RFBand(const ArtePointer<GWAL>& g){
  t_xmin = g->xmin;
  t_xmax = g->xmax;
  t_dx = fabs(t_xmax-t_xmin);
  t_ymin = g->ymin;
  t_ymax = g->ymax;
  t_dy = fabs(t_ymax-t_ymin);
  t_zmin = g->zmin;
  t_zmax = g->zmax;
  t_dz = fabs(t_zmax-t_zmin);
  //apply rotation matrix to unit vector along z
  float f[3] = {g->rot[2],g->rot[5],g->rot[8]};
  int k = (dy()<dx()) ? 1 : 0;
  t_slope = f[k]/f[2];
  t_zstop = t_zmin + dz()*fabs(f[2]);
}
#endif

float L3_RFBand::pathLength(const L3_Track &t) const {
  float result = -1.0;
  if( dy() < dx() ){
    //horizontal bands
    float d = slope() - t.my(); 
    if( fabs(d)==0 ) return result;
    float z = (t.y()-ymin()-t.refz()*t.my()+zmin()*slope())/d;
    if( z<zmin() || z>zstop() ) return result;
    float x = t.x() + (z-t.refz())*t.mx();
    if( x<xmin() || x>xmax() ) return result;
    result = dy()/fabs(t.my()); //sin(x) = tan(x) = x
  } else {
    //vertical bands
    float d = slope() - t.mx(); 
    if( fabs(d)==0 ) return result;
    float z = (t.x()-xmin()-t.refz()*t.mx()+zmin()*slope())/d;
    if( z<zmin() || z>zstop() ) return result;
    float y = t.y() + (z-t.refz())*t.my();
    if( y<ymin() || y>ymax() ) return result;
    result = dx()/fabs(t.mx()); //sin(x) = tan(x) = x
  }
  return result;
}

vector<float> L3_RFBand::crossingPoint(const L3_Track &t) const {
  vector<float> result;
  if( dy() < dx() ){
    //horizontal bands
    float d = slope() - t.my(); 
    if( fabs(d)==0 ) return result;
    float z = (t.y()-ymin()-t.refz()*t.my()+zmin()*slope())/d;
    if( z<zmin() || z>zstop() ) return result;
    float x = t.x() + (z-t.refz())*t.mx();
    if( x<xmin() || x>xmax() ) return result;
    result.push_back(x); 
    result.push_back(ymin()); 
    result.push_back(z); 
  } else {
    //vertical bands
    float d = slope() - t.mx(); 
    if( fabs(d)==0 ) return result;
    float z = (t.x()-xmin()-t.refz()*t.mx()+zmin()*slope())/d;
    if( z<zmin() || z>zstop() ) return result;
    float y = t.y() + (z-t.refz())*t.my();
    if( y<ymin() || y>ymax() ) return result;
    result.push_back(xmin()); 
    result.push_back(y); 
    result.push_back(z); 
  }
  return result;
}

L3_RFBand& L3_RFBand::operator = (const L3_RFBand& a){
  t_dx = a.t_dx;   
  t_dy = a.t_dy; 
  t_dz = a.t_dz;
  t_xmin = a.t_xmin;
  t_xmax = a.t_xmax;
  t_ymin = a.t_ymin;
  t_ymax = a.t_ymax;
  t_zmin = a.t_zmin;
  t_zmax = a.t_zmax;
  t_slope = a.t_slope;
  t_zstop = a.t_zstop;
  return *this;
}

