#ifndef _L3_WINDOW_HH_
#define _L3_WINDOW_HH_
// *****************************************************************************
//
// source:    L3_Window.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************

#include "L3_Defines.hh"
#include <cmath>
#include <iostream>
#include <CLHEP/Vector/ThreeVector.h>

class L3_Window {
public:
  inline L3_Window();
  inline L3_Window(const L3_Window&);
  inline L3_Window(const Hep3Vector&, const float&, const float&);
  inline L3_Window(const float&, const float&,
		   const float&, const float&,
		   const float&);
  inline L3_Window& operator = (const L3_Window&);
  inline int overlap(const L3_Window &anotherwin) const;

  inline int operator == (const L3_Window&) const;
  inline int operator != (const L3_Window&) const;

  inline float xmin() const;
  inline float xmax() const;
  inline float ymin() const;
  inline float ymax() const;
  inline float x() const;
  inline float y() const;
  inline float z() const;
  inline float dx() const;
  inline float dy() const;
  inline Hep3Vector PredPunkt() const;
  inline int isInside(float x,float y) const;
  inline int inX(float x) const;
  inline int inY(float y) const;

private:
  float the_xmin;
  float the_xmax;
  float the_ymin;
  float the_ymax;
  Hep3Vector the_prediction;
};

inline ostream& operator<<(ostream&,const L3_Window&);

#include "L3_Window.icc"

#endif
