#ifndef __OTR_ACCESS_DECODE_H__
#define __OTR_ACCESS_DECODE_H__

#ifdef __cplusplus
extern "C" 
{
#endif

  int otr_gede_status(int igede_, int mono, int wire);
    /********************************************************************
      Returns status of wire from wire status using OTrDecode structure.

      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table,
      mono  - number of monolayer (0..3)
      wire  - number of wire (0..31)
      ********************************************************************/
  
  int otr_gede_status_(int *igede, int *mono, int *iwire);
    /********************************************************************
      Fortran integer function:
      call otr_gede_status_(igede, mono, iwire)
      
      Returns status of wire from wire status using OTrDecode structure.
      
      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table,
      mono  - number of monolayer (1..4)
      iwire - number of wire (1..32)
      ********************************************************************/

  int otr_gede_channel(int igede_, int mono, int wire);
    /********************************************************************
      Returns channel number using OTrDecode structure.
      
      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table,
      mono  - number of monolayer (0..3)
      wire  - number of wire (0..31)
      ********************************************************************/
  
  int otr_gede_channel_(int *igede, int *mono, int *iwire);
    /********************************************************************
      Fortran integer function:
      integer otr_gede_channel,igede,mono,iwire
      channel = otr_gede_channel(igede, mono, iwire)
      
      Returns channel of wire using OTrDecode structure.
      
      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table,
      mono  - number of monolayer (1..4)
      iwire - number of wire (1..32)
      ********************************************************************/
  
  int otr_gede_channels(int igede_);
    /********************************************************************
      Returns number of channels for given superlayer/layer/sector
      using OTrDecode structure.
      
      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table
      ********************************************************************/
  
  int otr_gede_channels_(int *igede);
    /********************************************************************
      Fortran integer function:
      integer nchan,igede
      nchan = otr_gede_channels(igede)
      
      igede - index in GEDE table
      ********************************************************************/

  int OTrDecode_SuperLayerNameToIndex(const char slname[4], unsigned char* slid) ;
      /************************************************************************/
      /* Converts superlayername of form "-t1" to OTrDecode superlayer index  */
      /************************************************************************/
  
  int OTrDecode_ChannelToWire(OTR_Decode* OTR, unsigned char slid, 
			      unsigned char layid, unsigned char secid, 
			      unsigned char asdid, unsigned char chanid, 
			      unsigned short* gedeid, unsigned char* wireid) ;
      /************************************************************************/
      /* Converts channel identified by (superlayer,layer,sector,asd,channel) */
      /* to wire (GEDE, wire). The wireindex is defined by wire = cell + 32*mono. */
      /************************************************************************/

#ifdef __cplusplus
}
#endif

#endif
